# -*- coding: utf_8 -*-
"""
Shared Functions for Suppression logic.

Module provide support for finding suppression
"""
import logging
from copy import copy, deepcopy

from django.views.decorators.http import require_http_methods

from mobsf.StaticAnalyzer.models import (
    StaticAnalyzerAndroid,
    StaticAnalyzerIOS,
)
from mobsf.DynamicAnalyzer.views.common.shared import (
    invalid_params,
    send_response,
)
from mobsf.MobSF.security import is_attack_pattern
from mobsf.MobSF.utils import (
    android_component,
    is_md5,
    python_dict,
    python_list,
)
from mobsf.StaticAnalyzer.models import (
    SuppressFindings,
)
from mobsf.MobSF.views.authentication import (
    login_required,
)
from mobsf.MobSF.views.authorization import (
    Permissions,
    permission_required,
)

logger = logging.getLogger(__name__)

HIGH = 'high'
WARNING = 'warning'
INFO = 'info'
SECURE = 'secure'
GOOD = 'good'
SUPPRESSED = 'suppressed'


def get_package(checksum):
    """Get package from checksum."""
    try:
        andpkg = StaticAnalyzerAndroid.objects.get(
            MD5=checksum)
        return andpkg.PACKAGE_NAME
    except Exception:
        pass
    try:
        iospkg = StaticAnalyzerIOS.objects.get(
            MD5=checksum)
        return iospkg.BUNDLE_ID
    except Exception:
        pass
    return None


# AJAX

@login_required
@require_http_methods(['POST'])
@permission_required(Permissions.SUPPRESS)
def suppress_by_rule_id(request, api=False):
    """Suppress finding by rule id."""
    data = {
        'status': 'failed',
        'message': 'Failed to suppress finding by rule id'}
    try:
        if api:
            checksum = request.POST['hash']
        else:
            checksum = request.POST['checksum']
        rule = request.POST['rule']
        stype = request.POST['type']
        type_check = stype in {'code', 'manifest'}
        if not is_md5(checksum):
            return invalid_params(api)
        package = get_package(checksum)
        if not package or is_attack_pattern(rule) or not type_check:
            return invalid_params(api)
        sup_config = SuppressFindings.objects.filter(
            PACKAGE_NAME=package, SUPPRESS_TYPE=stype)
        if sup_config.exists():
            # Update Record
            sup_rules = set(
                python_list(sup_config[0].SUPPRESS_RULE_ID))
            if rule not in sup_rules:
                sup_rules.add(rule)
                sup_config.update(SUPPRESS_RULE_ID=list(sup_rules))
        else:
            # Create Record
            values = {
                'PACKAGE_NAME': package,
                'SUPPRESS_RULE_ID': [rule],
                'SUPPRESS_FILES': {},
                'SUPPRESS_TYPE': stype,
            }
            SuppressFindings.objects.create(**values)
        return send_response({'status': 'ok'}, api)
    except Exception:
        logger.exception('Error suppressing finding by rule id')
    return send_response(data, api)

# AJAX


@login_required
@require_http_methods(['POST'])
@permission_required(Permissions.SUPPRESS)
def suppress_by_files(request, api=False):
    """Suppress finding by files."""
    data = {
        'status': 'failed',
        'message': 'Failed to suppress finding by files'}
    try:
        if api:
            checksum = request.POST['hash']
        else:
            checksum = request.POST['checksum']
        rule = request.POST['rule']
        files_to_suppress = []
        old_files = []
        if not is_md5(checksum):
            return invalid_params(api)
        package = get_package(checksum)
        if not package or is_attack_pattern(rule):
            return invalid_params(api)
        sup_config = SuppressFindings.objects.filter(
            PACKAGE_NAME=package, SUPPRESS_TYPE='code')
        # Do Lookups
        android_static_db = StaticAnalyzerAndroid.objects.filter(
            MD5=checksum)
        ios_static_db = StaticAnalyzerIOS.objects.filter(
            MD5=checksum)
        if android_static_db.exists():
            code_res = python_dict(android_static_db[0].CODE_ANALYSIS)
        elif ios_static_db.exists():
            code_res = python_dict(ios_static_db[0].CODE_ANALYSIS)
        else:
            return send_response(data, api)
        files_to_suppress = list(code_res[rule]['files'].keys())
        if sup_config.exists():
            # Update Record
            old = python_dict(sup_config[0].SUPPRESS_FILES)
            # Empty dict or rule key not already present
            if not old or rule not in old:
                old[rule] = files_to_suppress
            else:
                old_files = old[rule]
                old[rule] = list(set(files_to_suppress + old_files))
            sup_config.update(SUPPRESS_FILES=old)
        else:
            # Create Record
            values = {
                'PACKAGE_NAME': package,
                'SUPPRESS_RULE_ID': [],
                'SUPPRESS_FILES': {rule: files_to_suppress},
                'SUPPRESS_TYPE': 'code',
            }
            SuppressFindings.objects.create(**values)
        return send_response({'status': 'ok'}, api)
    except Exception:
        logger.exception('Error suppressing finding by files')
    return send_response(data, api)

# AJAX


@login_required
@require_http_methods(['POST'])
def list_suppressions(request, api=False):
    """List Suppression Rules."""
    data = {
        'status': 'failed',
        'message': 'Failed to list suppression rules'}
    try:
        if api:
            checksum = request.POST['hash']
        else:
            checksum = request.POST['checksum']
        if not is_md5(checksum):
            return invalid_params(api)
        package = get_package(checksum)
        if not package:
            return invalid_params(api)

        data = []
        all_configs = SuppressFindings.objects.filter(
            PACKAGE_NAME=package)
        if all_configs.exists():
            data = list(all_configs.values())
            for i in data:
                i['SUPPRESS_RULE_ID'] = python_list(i['SUPPRESS_RULE_ID'])
                i['SUPPRESS_FILES'] = python_dict(i['SUPPRESS_FILES'])
        return send_response(
            {'status': 'ok', 'message': data},
            api)
    except Exception:
        logger.exception('Error listing suppression rules')
    return send_response(data, api)


# AJAX


@login_required
@require_http_methods(['POST'])
@permission_required(Permissions.DELETE)
def delete_suppression(request, api=False):
    """Delete suppression rule."""
    data = {
        'status': 'failed',
        'message': 'Failed to delete suppression rule'}
    try:
        if api:
            checksum = request.POST['hash']
        else:
            checksum = request.POST['checksum']
        rule = request.POST['rule']
        kind = request.POST['kind']
        stype = request.POST['type']
        type_check = stype in {'code', 'manifest'}
        if not is_md5(checksum):
            return invalid_params(api)
        package = get_package(checksum)
        if not package or is_attack_pattern(rule) or not type_check:
            return invalid_params(api)
        sup_config = SuppressFindings.objects.filter(
            PACKAGE_NAME=package, SUPPRESS_TYPE=stype)
        if sup_config.exists():
            if kind == 'rule':
                sup_rules = set(
                    python_list(sup_config[0].SUPPRESS_RULE_ID))
                if rule in sup_rules:
                    sup_rules.remove(rule)
                    sup_config.update(SUPPRESS_RULE_ID=list(sup_rules))
            elif kind == 'file':
                files_config = python_dict(sup_config[0].SUPPRESS_FILES)
                if rule in files_config:
                    del files_config[rule]
                    sup_config.update(SUPPRESS_FILES=files_config)
        return send_response({'status': 'ok'}, api)
    except Exception:
        logger.exception('Error deleting suppression rule')
    return send_response(data, api)


def process_suppression(data, package):
    """Process all suppression for code."""
    filtered = {}
    summary = {HIGH: 0, WARNING: 0, INFO: 0,
               SECURE: 0, SUPPRESSED: 0}
    if len(data) == 0:
        return {
            'findings': data,
            'summary': {},
        }
    filters = SuppressFindings.objects.filter(
        PACKAGE_NAME=package,
        SUPPRESS_TYPE='code')
    if not filters.exists():
        cleaned = data
    else:
        # Priority to rules
        filter_rules = python_list(filters[0].SUPPRESS_RULE_ID)
        if filter_rules:
            for k in data:
                if k not in filter_rules:
                    filtered[k] = data[k]
                else:
                    summary[SUPPRESSED] += 1
        else:
            filtered = deepcopy(data)

        # Process by files
        filter_files = python_dict(filters[0].SUPPRESS_FILES)
        cleaned = copy(filtered)
        if filter_files:
            for k in filtered:
                if k not in filter_files.keys():
                    continue
                for rem_file in filter_files[k]:
                    if rem_file in filtered[k]['files']:
                        del filtered[k]['files'][rem_file]
                        summary[SUPPRESSED] += 1
                # Remove rule_id with no files
                if len(filtered[k]['files']) == 0:
                    del cleaned[k]
    for v in cleaned.values():
        if 'severity' in v:
            # iOS binary code
            sev = v['severity']
        else:
            sev = v['metadata']['severity']
        if sev == HIGH:
            summary[HIGH] += 1
        elif sev == WARNING:
            summary[WARNING] += 1
        elif sev == INFO:
            summary[INFO] += 1
        elif sev == GOOD or sev == SECURE:
            summary[SECURE] += 1
    return {
        'findings': cleaned,
        'summary': summary,
    }


def process_suppression_manifest(data, package):
    """Process all suppression for manifest."""
    filtered = []
    summary = {HIGH: 0, WARNING: 0, INFO: 0, SUPPRESSED: 0}
    filters = SuppressFindings.objects.filter(
        PACKAGE_NAME=package,
        SUPPRESS_TYPE='manifest')
    if not filters.exists():
        filtered = data
    else:
        filter_rules = python_list(filters[0].SUPPRESS_RULE_ID)
        if filter_rules:
            for k in data:
                rule = k['rule']
                title = k['title']
                dynamic_rule = f'{android_component(title)}{rule}'
                if dynamic_rule not in filter_rules:
                    filtered.append(k)
                else:
                    summary[SUPPRESSED] += 1
        else:
            filtered = data
    for i in filtered:
        if i['severity'] == HIGH:
            summary[HIGH] += 1
        elif i['severity'] == WARNING:
            summary[WARNING] += 1
        elif ['severity'] == INFO:
            summary[INFO] += 1
    return {
        'manifest_findings': filtered,
        'manifest_summary': summary,
    }



# -*- coding: utf_8 -*-
"""Available Actions."""
import logging
import os
import re

from django.conf import settings
from django.views.decorators.http import require_http_methods

from mobsf.DynamicAnalyzer.views.common.shared import (
    invalid_params,
    send_response,
)
from mobsf.DynamicAnalyzer.views.android.operations import (
    get_package_name,
)
from mobsf.DynamicAnalyzer.views.android.environment import (
    Environment,
)
from mobsf.DynamicAnalyzer.views.android.tests_xposed import (
    download_xposed_log,
)
from mobsf.DynamicAnalyzer.views.android.tests_tls import (
    run_tls_tests,
)
from mobsf.DynamicAnalyzer.tools.webproxy import (
    get_http_tools_url,
    stop_httptools,
)
from mobsf.MobSF.security import cmd_injection_check
from mobsf.MobSF.utils import (
    is_md5,
    python_list,
)
from mobsf.StaticAnalyzer.models import StaticAnalyzerAndroid
from mobsf.MobSF.views.authentication import (
    login_required,
)
from mobsf.MobSF.views.authorization import (
    Permissions,
    permission_required,
)

logger = logging.getLogger(__name__)


# AJAX


@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def start_activity(request, api=False):
    """Launch a specific activity."""
    try:
        env = Environment()
        activity = request.POST['activity']
        md5_hash = request.POST['hash']

        valid_md5 = is_md5(md5_hash)
        valid_act = re.match(r'^[\w]+(\.[\w]+)*$', activity)
        if not valid_act or not valid_md5:
            return invalid_params(api)

        app_dir = os.path.join(settings.UPLD_DIR, md5_hash + '/')
        screen_dir = os.path.join(app_dir, 'screenshots-apk/')
        if not os.path.exists(screen_dir):
            os.makedirs(screen_dir)
        logger.info('Launching Activity - %s', activity)
        outfile = ('{}act-{}.png'.format(screen_dir, activity))
        static_android_db = StaticAnalyzerAndroid.objects.get(
            MD5=md5_hash)
        package = static_android_db.PACKAGE_NAME
        env.launch_n_capture(package, activity, outfile)
        data = {'status': 'ok'}
    except Exception as exp:
        logger.exception('Start Activity')
        data = {'status': 'failed', 'message': str(exp)}
    return send_response(data, api)

# AJAX


@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def start_deeplink(request, api=False):
    """Launch a specific deeplink."""
    try:
        env = Environment()
        url = request.POST['url']
        md5_hash = request.POST['hash']

        valid_md5 = is_md5(md5_hash)
        if cmd_injection_check(url) or not valid_md5:
            return invalid_params(api)
        env.adb_command(
            ['am', 'start', '-a',
             'android.intent.action.VIEW',
             '-d', url], True)
        data = {'status': 'ok'}
    except Exception as exp:
        logger.exception('Start Activity')
        data = {'status': 'failed', 'message': str(exp)}
    return send_response(data, api)

# AJAX


@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def activity_tester(request, api=False):
    """Exported & non exported activity Tester."""
    data = {}
    try:
        env = Environment()
        test = request.POST['test']
        md5_hash = request.POST['hash']
        if not is_md5(md5_hash):
            return invalid_params(api)
        app_dir = os.path.join(settings.UPLD_DIR, md5_hash + '/')
        screen_dir = os.path.join(app_dir, 'screenshots-apk/')
        if not os.path.exists(screen_dir):
            os.makedirs(screen_dir)
        static_android_db = StaticAnalyzerAndroid.objects.get(
            MD5=md5_hash)
        package = static_android_db.PACKAGE_NAME
        iden = ''
        if test == 'exported':
            iden = 'Exported '
            logger.info('Exported activity tester')
            activities = python_list(static_android_db.EXPORTED_ACTIVITIES)
        else:
            logger.info('Activity tester')
            activities = python_list(static_android_db.ACTIVITIES)
        logger.info('Fetching %sactivities for %s', iden, package)
        if not activities:
            msg = 'No {}Activities found'.format(iden)
            logger.info(msg)
            data = {'status': 'failed',
                    'message': msg}
            return send_response(data, api)
        act_no = 0
        logger.info('Starting %sActivity Tester...', iden)
        logger.info('%s %sActivities Identified',
                    str(len(activities)), iden)
        for activity in activities:
            act_no += 1
            logger.info(
                'Launching %sActivity - %s. %s',
                iden,
                str(act_no),
                activity)
            if test == 'exported':
                file_iden = 'expact'
            else:
                file_iden = 'act'
            outfile = ('{}{}-{}.png'.format(
                screen_dir,
                file_iden,
                act_no))
            env.launch_n_capture(package, activity, outfile)
        data = {'status': 'ok'}
    except Exception as exp:
        logger.exception('%sActivity tester', iden)
        data = {'status': 'failed', 'message': str(exp)}
    return send_response(data, api)

# AJAX


@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def download_data(request, api=False):
    """Download Application Data from Device."""
    logger.info('Downloading app data')
    data = {}
    try:
        env = Environment()
        md5_hash = request.POST['hash']
        if not is_md5(md5_hash):
            return invalid_params(api)
        package = get_package_name(md5_hash)
        if not package:
            data = {'status': 'failed',
                    'message': 'App details not found in database'}
            return send_response(data, api)
        apk_dir = os.path.join(settings.UPLD_DIR, md5_hash + '/')
        stop_httptools(get_http_tools_url(request))
        files_loc = '/data/local/'
        logger.info('Archiving files created by app')
        env.adb_command(['tar', '-cvf', files_loc + package + '.tar',
                         '/data/data/' + package + '/'], True)
        logger.info('Downloading Archive')
        env.adb_command(['pull', files_loc + package + '.tar',
                         apk_dir + package + '.tar'])
        logger.info('Stopping ADB server')
        env.adb_command(['kill-server'])
        data = {'status': 'ok'}
    except Exception as exp:
        logger.exception('Downloading application data')
        data = {'status': 'failed', 'message': str(exp)}
    return send_response(data, api)

# AJAX


@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def collect_logs(request, api=False):
    """Collecting Data and Cleanup."""
    logger.info('Collecting Data and Cleaning Up')
    data = {}
    try:
        env = Environment()
        md5_hash = request.POST['hash']
        if not is_md5(md5_hash):
            return invalid_params(api)
        package = get_package_name(md5_hash)
        if not package:
            data = {'status': 'failed',
                    'message': 'App details not found in database'}
            return send_response(data, api)
        apk_dir = os.path.join(settings.UPLD_DIR, md5_hash + '/')
        lout = os.path.join(apk_dir, 'logcat.txt')
        dout = os.path.join(apk_dir, 'dump.txt')
        logger.info('Downloading logcat logs')
        logcat = env.adb_command(['logcat',
                                  '-d',
                                  package + ':V',
                                  '*:*'])
        with open(lout, 'wb') as flip:
            flip.write(logcat)
        logger.info('Downloading dumpsys logs')
        dumpsys = env.adb_command(['dumpsys'], True)
        with open(dout, 'wb') as flip:
            flip.write(dumpsys)
        if env.get_android_version() < 5:
            download_xposed_log(apk_dir)
        env.adb_command(['am', 'force-stop', package], True)
        logger.info('Stopping app')
        # Unset Global Proxy
        env.unset_global_proxy()
        data = {'status': 'ok'}
    except Exception as exp:
        logger.exception('Data Collection & Clean Up failed')
        data = {'status': 'failed', 'message': str(exp)}
    return send_response(data, api)

# AJAX


@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def tls_tests(request, api=False):
    """Perform TLS tests."""
    logger.info('Running TLS/SSL Security tests')
    data = {}
    package = None
    try:
        test_duration = 25
        test_package = 'tls_tests'
        env = Environment()
        md5_hash = request.POST['hash']
        if not is_md5(md5_hash):
            return invalid_params(api)
        package = get_package_name(md5_hash)
        if not package:
            data = {'status': 'failed',
                    'message': 'App details not found in database'}
            return send_response(data, api)
        res = run_tls_tests(
            request,
            md5_hash,
            env,
            package,
            test_package,
            test_duration,
        )
        data = {'status': 'ok', 'tls_tests': res}
    except Exception as exp:
        logger.exception('Checking Application Security in Transit')
        data = {'status': 'failed', 'message': str(exp)}
    finally:
        logger.info('Test Completed. Resuming HTTPS Proxy')
        env.configure_proxy(package, request)
    return send_response(data, api)



# -*- coding: utf_8 -*-
"""Frida tests for iOS."""
from threading import Thread
import logging

from django.views.decorators.http import require_http_methods

from mobsf.DynamicAnalyzer.views.ios.frida_core import (
    Frida,
)
from mobsf.DynamicAnalyzer.views.common.shared import (
    invalid_params,
    send_response,
)
from mobsf.MobSF.security import is_attack_pattern
from mobsf.DynamicAnalyzer.views.ios.corellium_apis import (
    CorelliumInstanceAPI,
    OK,
)
from mobsf.MobSF.utils import (
    common_check,
    is_md5,
    strict_package_check,
)
from mobsf.MobSF.views.authentication import (
    login_required,
)
from mobsf.MobSF.views.authorization import (
    Permissions,
    permission_required,
)

logger = logging.getLogger(__name__)

# AJAX


@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def ios_instrument(request, api=False):
    """Instrument app with frida."""
    data = {
        'status': 'failed',
        'message': ''}
    try:
        action = request.POST.get('frida_action', 'spawn')
        pid = request.POST.get('pid')
        new_bundle_id = request.POST.get('new_bundle_id')
        instance_id = request.POST['instance_id']
        bundle_id = request.POST['bundle_id']
        md5_hash = request.POST['hash']
        default_hooks = request.POST['default_hooks']
        dump_hooks = request.POST['dump_hooks']
        auxiliary_hooks = request.POST['auxiliary_hooks']
        code = request.POST['frida_code']

        failed = common_check(instance_id)
        if failed:
            return send_response(failed, api)
        if not strict_package_check(bundle_id):
            data['message'] = 'Invalid iOS Bundle id'
            return send_response(data, api)
        if new_bundle_id and not strict_package_check(new_bundle_id):
            data['message'] = 'Invalid iOS Bundle id'
            return send_response(data, api)
        ci = CorelliumInstanceAPI(instance_id)

        # Fill extras
        extras = {}
        class_name = request.POST.get('class_name')
        if class_name:
            extras['class_name'] = class_name.strip()
        class_search = request.POST.get('class_search')
        if class_search:
            extras['class_search'] = class_search.strip()
        method_search = request.POST.get('method_search')
        if method_search:
            extras['method_search'] = method_search.strip()
        cls_trace = request.POST.get('class_trace')
        if cls_trace:
            extras['class_trace'] = cls_trace.strip()
        if (is_attack_pattern(default_hooks)
                or is_attack_pattern(dump_hooks)
                or is_attack_pattern(auxiliary_hooks)
                or not is_md5(md5_hash)):
            return invalid_params(api)

        frida_obj = Frida(
            ci.get_ssh_connection_string(),
            md5_hash,
            bundle_id,
            default_hooks.split(','),
            dump_hooks.split(','),
            auxiliary_hooks.split(','),
            extras,
            code)
        if action == 'spawn':
            logger.info('Starting Instrumentation')
            frida_obj.spawn()
        elif action == 'ps':
            logger.info('Enumerating running applications')
            data['message'] = frida_obj.ps()
        elif action == 'get':
            # Get injected Frida script.
            data['message'] = frida_obj.get_script(nolog=True)
        if action in ('spawn', 'session'):
            if pid and pid.isdigit():
                # Attach to a different pid/bundle id
                args = (int(pid), new_bundle_id)
                logger.info('Attaching to %s [PID: %s]', new_bundle_id, pid)
            else:
                # Injecting to existing session/spawn
                if action == 'session':
                    logger.info('Injecting to existing frida session')
                args = (None, None)
            Thread(target=frida_obj.session, args=args, daemon=True).start()
        data['status'] = OK
    except Exception as exp:
        logger.exception('Frida Instrumentation failed')
        data['message'] = str(exp)
    return send_response(data, api)



"""Frida Core Class for iOS Device."""
import logging
import time
import sys

import frida

from mobsf.DynamicAnalyzer.views.ios.frida_core import Frida
from mobsf.DynamicAnalyzer.views.ios.device.environment import IOSEnvironment

logger = logging.getLogger(__name__)
_PID = None
_SPAWN_LOG_MSG = 'Spawned %s with PID %s'


class FridaIOSDevice(Frida):
    """Frida iOS Device for Dynamic Analysis."""

    def __init__(
            self, ios_device, ssh_string, app_hash, bundle_id,
            defaults, dump, auxiliary, extras, code):
        """Initialize."""
        super().__init__(
            ssh_string, app_hash, bundle_id, defaults, dump, auxiliary, extras, code)
        self.connector = ios_device.connector
        self.env = IOSEnvironment(ios_device)
        self.api = None
        # USB path: iproxy maps localhost:37042 -> device:27042
        #   MobSF starts this automatically via _setup_usb_port_forward().
        #   Frida connects to localhost:37042.
        #
        # WiFi / Docker path: Frida connects directly to the device host
        #   on the default Frida port (27042). No local port mapping is
        #   created by MobSF. When running in Docker/Colima, run iproxy
        #   on the macOS host before starting MobSF:
        #     iproxy -s 0.0.0.0 2222:22 27042:27042 -u <udid>
        #   Then set MOBSF_IOS_ANALYZER_IDENTIFIERS=host.lima.internal:2222
        #   (Colima) or host.docker.internal:2222 (Docker Desktop).
        if self.connector.connection_type == 'usb':
            self.frida_device = frida.get_device_manager().add_remote_device(
                f'{self.connector.host}:37042')
        else:
            self.frida_device = frida.get_device_manager().add_remote_device(
                self.connector.host)

    def run_app(self):
        """Run the app with frida."""
        pid = None
        try:
            try:
                pid = self.frida_device.spawn([self.bundle_id])
                self.frida_device.resume(pid)
                logger.info(_SPAWN_LOG_MSG, self.bundle_id, pid)
                return pid
            except frida.NotSupportedError:
                logger.error(self.not_supported_text)
                logger.info(_SPAWN_LOG_MSG, self.bundle_id, pid)
                return pid
            except frida.ServerNotRunningError:
                self.env.start_frida_server()
                time.sleep(1)
                pid = self.frida_device.spawn([self.bundle_id])
                self.frida_device.resume(pid)
                logger.info(_SPAWN_LOG_MSG, self.bundle_id, pid)
                return pid
        except frida.TimedOutError:
            logger.error('Timed out while waiting for device to appear')
        except frida.ServerNotRunningError:
            logger.error('Frida Server is not running')
        except frida.NotSupportedError:
            logger.error(self.not_supported_text)
        except (frida.ProcessNotFoundError,
                frida.ProcessNotRespondingError,
                frida.TransportError,
                frida.InvalidOperationError):
            pass
        except Exception:
            logger.exception('Failed to run app')
        return None

    def spawn(self):
        """Connect to Frida Server and spawn the app."""
        global _PID
        try:
            try:
                self.clean_up()
                _PID = self.frida_device.spawn([self.bundle_id])
            except frida.NotSupportedError:
                logger.error(self.not_supported_text)
                return
            except frida.ServerNotRunningError:
                self.env.start_frida_server()
                _PID = None
            if not _PID:
                _PID = self.frida_device.spawn([self.bundle_id])
            logger.info(_SPAWN_LOG_MSG, self.bundle_id, _PID)
        except frida.TimedOutError:
            logger.error('Timed out while waiting for device to appear')
        except frida.ServerNotRunningError:
            logger.error('Frida Server is not running')
        except frida.NotSupportedError:
            logger.error(self.not_supported_text)
            return
        except (frida.ProcessNotFoundError,
                frida.ProcessNotRespondingError,
                frida.TransportError,
                frida.InvalidOperationError):
            pass
        except Exception:
            logger.exception('Error Connecting to Frida Server')

    def session(self, pid, bundle_id):
        """Use existing session to inject frida scripts."""
        global _PID
        try:
            try:
                if pid and bundle_id:
                    _PID = pid
                    self.bundle_id = bundle_id
                try:
                    front = self.frida_device.get_frontmost_application()
                    if front or front.pid != _PID:
                        # Not the front most app.
                        # Get the pid of the front most app
                        logger.warning('Front most app has PID %s', front.pid)
                        _PID = front.pid
                except Exception:
                    pass
                self.frida_device.resume(_PID)
                time.sleep(2)
                session = self.frida_device.attach(_PID)
            except frida.NotSupportedError:
                logger.error(self.not_supported_text)
                return
            except Exception:
                logger.warning('Cannot attach to pid, spawning again.')
                self.spawn()
                self.frida_device.resume(_PID)
                time.sleep(2)
                session = self.frida_device.attach(_PID)
            if session and self.frida_device and _PID:
                script = session.create_script(self.get_script())
                script.on('message', self.frida_response)
                script.load()
                self.api = script.exports_sync
                self.api_handler(self.api)
                sys.stdin.read()
                script.unload()
                session.detach()
        except frida.NotSupportedError:
            logger.error(self.not_supported_text)
        except (frida.ProcessNotFoundError,
                frida.ProcessNotRespondingError,
                frida.TransportError,
                frida.InvalidOperationError):
            pass
        except Exception:
            logger.exception('Error Connecting to Frida Server')

    def ps(self):
        """Get running process pid."""
        ps_dict = []
        try:
            try:
                processes = self.frida_device.enumerate_applications(scope='minimal')
            except frida.ServerNotRunningError:
                self.env.start_frida_server()
                processes = self.frida_device.enumerate_applications(scope='minimal')
            if self.frida_device and processes:
                for process in processes:
                    if process.pid != 0:
                        ps_dict.append({
                            'pid': process.pid,
                            'name': process.name,
                            'identifier': process.identifier,
                        })
        except Exception:
            logger.exception('Failed to enumerate running applications')
        return ps_dict



# -*- coding: utf_8 -*-
"""iOS helpers."""
import logging
from threading import Thread
from pathlib import Path

from django.conf import settings

from mobsf.DynamicAnalyzer.tools.webproxy import (
    get_http_tools_url,
    start_proxy,
    stop_httptools,
)
from mobsf.MobSF.utils import (
    get_md5,
    python_dict,
)
from mobsf.DynamicAnalyzer.views.ios.corellium_ssh import (
    ssh_jumphost_reverse_port_forward,
)
from mobsf.StaticAnalyzer.models import StaticAnalyzerIOS


logger = logging.getLogger(__name__)


def get_local_ipa_list():
    """Get local ipa list."""
    scan_apps = []
    ipas = StaticAnalyzerIOS.objects.filter(
        FILE_NAME__endswith='.ipa')
    for ipa in reversed(ipas):
        bundle_hash = get_md5(ipa.BUNDLE_ID.encode('utf-8'))
        frida_dump = Path(
            settings.UPLD_DIR) / bundle_hash / 'mobsf_dump_file.txt'
        macho = python_dict(ipa.MACHO_ANALYSIS)
        encrypted = False
        if (macho
                and macho.get('encrypted')
                and macho.get('encrypted').get('is_encrypted')):
            encrypted = macho['encrypted']['is_encrypted']
        temp_dict = {
            'MD5': ipa.MD5,
            'APP_NAME': ipa.APP_NAME,
            'APP_VERSION': ipa.APP_VERSION,
            'FILE_NAME': ipa.FILE_NAME,
            'BUNDLE_ID': ipa.BUNDLE_ID,
            'BUNDLE_HASH': bundle_hash,
            'ENCRYPTED': encrypted,
            'DYNAMIC_REPORT_EXISTS': frida_dump.exists(),
        }
        scan_apps.append(temp_dict)
    return scan_apps


def get_ios_model_mapping():
    """Get iOS model mapping."""
    return {
        # iPhone Simulator
        'i386': 'iPhone Simulator',
        'x86_64': 'iPhone Simulator',
        'arm64': 'iPhone Simulator',

        # iPhone Models
        'iPhone1,1': 'iPhone',
        'iPhone1,2': 'iPhone 3G',
        'iPhone2,1': 'iPhone 3GS',
        'iPhone3,1': 'iPhone 4',
        'iPhone3,2': 'iPhone 4 GSM Rev A',
        'iPhone3,3': 'iPhone 4 CDMA',
        'iPhone4,1': 'iPhone 4S',
        'iPhone5,1': 'iPhone 5 (GSM)',
        'iPhone5,2': 'iPhone 5 (GSM+CDMA)',
        'iPhone5,3': 'iPhone 5C (GSM)',
        'iPhone5,4': 'iPhone 5C (Global)',
        'iPhone6,1': 'iPhone 5S (GSM)',
        'iPhone6,2': 'iPhone 5S (Global)',
        'iPhone7,1': 'iPhone 6 Plus',
        'iPhone7,2': 'iPhone 6',
        'iPhone8,1': 'iPhone 6s',
        'iPhone8,2': 'iPhone 6s Plus',
        'iPhone8,4': 'iPhone SE (GSM)',
        'iPhone9,1': 'iPhone 7',
        'iPhone9,2': 'iPhone 7 Plus',
        'iPhone9,3': 'iPhone 7',
        'iPhone9,4': 'iPhone 7 Plus',
        'iPhone10,1': 'iPhone 8',
        'iPhone10,2': 'iPhone 8 Plus',
        'iPhone10,3': 'iPhone X Global',
        'iPhone10,4': 'iPhone 8',
        'iPhone10,5': 'iPhone 8 Plus',
        'iPhone10,6': 'iPhone X GSM',
        'iPhone11,2': 'iPhone XS',
        'iPhone11,4': 'iPhone XS Max',
        'iPhone11,6': 'iPhone XS Max Global',
        'iPhone11,8': 'iPhone XR',
        'iPhone12,1': 'iPhone 11',
        'iPhone12,3': 'iPhone 11 Pro',
        'iPhone12,5': 'iPhone 11 Pro Max',
        'iPhone12,8': 'iPhone SE 2nd Gen',
        'iPhone13,1': 'iPhone 12 Mini',
        'iPhone13,2': 'iPhone 12',
        'iPhone13,3': 'iPhone 12 Pro',
        'iPhone13,4': 'iPhone 12 Pro Max',
        'iPhone14,2': 'iPhone 13 Pro',
        'iPhone14,3': 'iPhone 13 Pro Max',
        'iPhone14,4': 'iPhone 13 Mini',
        'iPhone14,5': 'iPhone 13',
        'iPhone14,6': 'iPhone SE 3rd Gen',
        'iPhone14,7': 'iPhone 14',
        'iPhone14,8': 'iPhone 14 Plus',
        'iPhone15,2': 'iPhone 14 Pro',
        'iPhone15,3': 'iPhone 14 Pro Max',
        'iPhone15,4': 'iPhone 15',
        'iPhone15,5': 'iPhone 15 Plus',
        'iPhone16,1': 'iPhone 15 Pro',
        'iPhone16,2': 'iPhone 15 Pro Max',
        'iPhone17,1': 'iPhone 16 Pro',
        'iPhone17,2': 'iPhone 16 Pro Max',
        'iPhone17,3': 'iPhone 16',
        'iPhone17,4': 'iPhone 16 Plus',
        'iPhone17,5': 'iPhone 16e',

        # iPod Models
        'iPod1,1': '1st Gen iPod',
        'iPod2,1': '2nd Gen iPod',
        'iPod3,1': '3rd Gen iPod',
        'iPod4,1': '4th Gen iPod',
        'iPod5,1': '5th Gen iPod',
        'iPod7,1': '6th Gen iPod',
        'iPod9,1': '7th Gen iPod',

        # iPad Models
        'iPad1,1': 'iPad',
        'iPad1,2': 'iPad 3G',
        'iPad2,1': '2nd Gen iPad',
        'iPad2,2': '2nd Gen iPad GSM',
        'iPad2,3': '2nd Gen iPad CDMA',
        'iPad2,4': '2nd Gen iPad New Revision',
        'iPad3,1': '3rd Gen iPad',
        'iPad3,2': '3rd Gen iPad CDMA',
        'iPad3,3': '3rd Gen iPad GSM',
        'iPad2,5': 'iPad mini',
        'iPad2,6': 'iPad mini GSM+LTE',
        'iPad2,7': 'iPad mini CDMA+LTE',
        'iPad3,4': '4th Gen iPad',
        'iPad3,5': '4th Gen iPad GSM+LTE',
        'iPad3,6': '4th Gen iPad CDMA+LTE',
        'iPad4,1': 'iPad Air (WiFi)',
        'iPad4,2': 'iPad Air (GSM+CDMA)',
        'iPad4,3': '1st Gen iPad Air (China)',
        'iPad4,4': 'iPad mini Retina (WiFi)',
        'iPad4,5': 'iPad mini Retina (GSM+CDMA)',
        'iPad4,6': 'iPad mini Retina (China)',
        'iPad4,7': 'iPad mini 3 (WiFi)',
        'iPad4,8': 'iPad mini 3 (GSM+CDMA)',
        'iPad4,9': 'iPad Mini 3 (China)',
        'iPad5,1': 'iPad mini 4 (WiFi)',
        'iPad5,2': '4th Gen iPad mini (WiFi+Cellular)',
        'iPad5,3': 'iPad Air 2 (WiFi)',
        'iPad5,4': 'iPad Air 2 (Cellular)',
        'iPad6,3': 'iPad Pro (9.7 inch, WiFi)',
        'iPad6,4': 'iPad Pro (9.7 inch, WiFi+LTE)',
        'iPad6,7': 'iPad Pro (12.9 inch, WiFi)',
        'iPad6,8': 'iPad Pro (12.9 inch, WiFi+LTE)',
        'iPad6,11': 'iPad (2017)',
        'iPad6,12': 'iPad (2017)',
        'iPad7,1': 'iPad Pro 2nd Gen (WiFi)',
        'iPad7,2': 'iPad Pro 2nd Gen (WiFi+Cellular)',
        'iPad7,3': 'iPad Pro 10.5-inch 2nd Gen',
        'iPad7,4': 'iPad Pro 10.5-inch 2nd Gen',
        'iPad7,5': 'iPad 6th Gen (WiFi)',
        'iPad7,6': 'iPad 6th Gen (WiFi+Cellular)',
        'iPad7,11': 'iPad 7th Gen 10.2-inch (WiFi)',
        'iPad7,12': 'iPad 7th Gen 10.2-inch (WiFi+Cellular)',
        'iPad8,1': 'iPad Pro 11 inch 3rd Gen (WiFi)',
        'iPad8,2': 'iPad Pro 11 inch 3rd Gen (1TB, WiFi)',
        'iPad8,3': 'iPad Pro 11 inch 3rd Gen (WiFi+Cellular)',
        'iPad8,4': 'iPad Pro 11 inch 3rd Gen (1TB, WiFi+Cellular)',
        'iPad8,5': 'iPad Pro 12.9 inch 3rd Gen (WiFi)',
        'iPad8,6': 'iPad Pro 12.9 inch 3rd Gen (1TB, WiFi)',
        'iPad8,7': 'iPad Pro 12.9 inch 3rd Gen (WiFi+Cellular)',
        'iPad8,8': 'iPad Pro 12.9 inch 3rd Gen (1TB, WiFi+Cellular)',
        'iPad8,9': 'iPad Pro 11 inch 4th Gen (WiFi)',
        'iPad8,10': 'iPad Pro 11 inch 4th Gen (WiFi+Cellular)',
        'iPad8,11': 'iPad Pro 12.9 inch 4th Gen (WiFi)',
        'iPad8,12': 'iPad Pro 12.9 inch 4th Gen (WiFi+Cellular)',
        'iPad11,1': 'iPad mini 5th Gen (WiFi)',
        'iPad11,2': 'iPad mini 5th Gen',
        'iPad11,3': 'iPad Air 3rd Gen (WiFi)',
        'iPad11,4': 'iPad Air 3rd Gen',
        'iPad11,6': 'iPad 8th Gen (WiFi)',
        'iPad11,7': 'iPad 8th Gen (WiFi+Cellular)',
        'iPad12,1': 'iPad 9th Gen (WiFi)',
        'iPad12,2': 'iPad 9th Gen (WiFi+Cellular)',
        'iPad14,1': 'iPad mini 6th Gen (WiFi)',
        'iPad14,2': 'iPad mini 6th Gen (WiFi+Cellular)',
        'iPad13,1': 'iPad Air 4th Gen (WiFi)',
        'iPad13,2': 'iPad Air 4th Gen (WiFi+Cellular)',
        'iPad13,4': 'iPad Pro 11 inch 5th Gen',
        'iPad13,5': 'iPad Pro 11 inch 5th Gen',
        'iPad13,6': 'iPad Pro 11 inch 5th Gen',
        'iPad13,7': 'iPad Pro 11 inch 5th Gen',
        'iPad13,8': 'iPad Pro 12.9 inch 5th Gen',
        'iPad13,9': 'iPad Pro 12.9 inch 5th Gen',
        'iPad13,10': 'iPad Pro 12.9 inch 5th Gen',
        'iPad13,11': 'iPad Pro 12.9 inch 5th Gen',
        'iPad13,16': 'iPad Air 5th Gen (WiFi)',
        'iPad13,17': 'iPad Air 5th Gen (WiFi+Cellular)',
        'iPad13,18': 'iPad 10th Gen',
        'iPad13,19': 'iPad 10th Gen',
        'iPad14,3': 'iPad Pro 11 inch 4th Gen',
        'iPad14,4': 'iPad Pro 11 inch 4th Gen',
        'iPad14,5': 'iPad Pro 12.9 inch 6th Gen',
        'iPad14,6': 'iPad Pro 12.9 inch 6th Gen',
        'iPad14,8': 'iPad Air 11 inch 6th Gen (WiFi)',
        'iPad14,9': 'iPad Air 11 inch 6th Gen (WiFi+Cellular)',
        'iPad14,10': 'iPad Air 13 inch 6th Gen (WiFi)',
        'iPad14,11': 'iPad Air 13 inch 6th Gen (WiFi+Cellular)',
        'iPad15,3': 'iPad Air 11-inch 7th Gen (WiFi)',
        'iPad15,4': 'iPad Air 11-inch 7th Gen (WiFi+Cellular)',
        'iPad15,5': 'iPad Air 13-inch 7th Gen (WiFi)',
        'iPad15,6': 'iPad Air 13-inch 7th Gen (WiFi+Cellular)',
        'iPad15,7': 'iPad 11th Gen (WiFi)',
        'iPad15,8': 'iPad 11th Gen (WiFi+Cellular)',
        'iPad16,1': 'iPad mini 7th Gen (WiFi)',
        'iPad16,2': 'iPad mini 7th Gen (WiFi+Cellular)',
        'iPad16,3': 'iPad Pro 11 inch 5th Gen',
        'iPad16,4': 'iPad Pro 11 inch 5th Gen',
        'iPad16,5': 'iPad Pro 12.9 inch 7th Gen',
        'iPad16,6': 'iPad Pro 12.9 inch 7th Gen',

        # Apple Watch Models
        'Watch1,1': 'Apple Watch 38mm case',
        'Watch1,2': 'Apple Watch 42mm case',
        'Watch2,6': 'Apple Watch Series 1 38mm case',
        'Watch2,7': 'Apple Watch Series 1 42mm case',
        'Watch2,3': 'Apple Watch Series 2 38mm case',
        'Watch2,4': 'Apple Watch Series 2 42mm case',
        'Watch3,1': 'Apple Watch Series 3 38mm case (GPS+Cellular)',
        'Watch3,2': 'Apple Watch Series 3 42mm case (GPS+Cellular)',
        'Watch3,3': 'Apple Watch Series 3 38mm case (GPS)',
        'Watch3,4': 'Apple Watch Series 3 42mm case (GPS)',
        'Watch4,1': 'Apple Watch Series 4 40mm case (GPS)',
        'Watch4,2': 'Apple Watch Series 4 44mm case (GPS)',
        'Watch4,3': 'Apple Watch Series 4 40mm case (GPS+Cellular)',
        'Watch4,4': 'Apple Watch Series 4 44mm case (GPS+Cellular)',
        'Watch5,1': 'Apple Watch Series 5 40mm case (GPS)',
        'Watch5,2': 'Apple Watch Series 5 44mm case (GPS)',
        'Watch5,3': 'Apple Watch Series 5 40mm case (GPS+Cellular)',
        'Watch5,4': 'Apple Watch Series 5 44mm case (GPS+Cellular)',
        'Watch5,9': 'Apple Watch SE 40mm case (GPS)',
        'Watch5,10': 'Apple Watch SE 44mm case (GPS)',
        'Watch5,11': 'Apple Watch SE 40mm case (GPS+Cellular)',
        'Watch5,12': 'Apple Watch SE 44mm case (GPS+Cellular)',
        'Watch6,1': 'Apple Watch Series 6 40mm case (GPS)',
        'Watch6,2': 'Apple Watch Series 6 44mm case (GPS)',
        'Watch6,3': 'Apple Watch Series 6 40mm case (GPS+Cellular)',
        'Watch6,4': 'Apple Watch Series 6 44mm case (GPS+Cellular)',
        'Watch6,6': 'Apple Watch Series 7 41mm case (GPS)',
        'Watch6,7': 'Apple Watch Series 7 45mm case (GPS)',
        'Watch6,8': 'Apple Watch Series 7 41mm case (GPS+Cellular)',
        'Watch6,9': 'Apple Watch Series 7 45mm case (GPS+Cellular)',
        'Watch6,10': 'Apple Watch SE 40mm case (GPS)',
        'Watch6,11': 'Apple Watch SE 44mm case (GPS)',
        'Watch6,12': 'Apple Watch SE 40mm case (GPS+Cellular)',
        'Watch6,13': 'Apple Watch SE 44mm case (GPS+Cellular)',
        'Watch6,14': 'Apple Watch Series 8 41mm case (GPS)',
        'Watch6,15': 'Apple Watch Series 8 45mm case (GPS)',
        'Watch6,16': 'Apple Watch Series 8 41mm case (GPS+Cellular)',
        'Watch6,17': 'Apple Watch Series 8 45mm case (GPS+Cellular)',
        'Watch6,18': 'Apple Watch Ultra',
        'Watch7,1': 'Apple Watch Series 9 41mm case (GPS)',
        'Watch7,2': 'Apple Watch Series 9 45mm case (GPS)',
        'Watch7,3': 'Apple Watch Series 9 41mm case (GPS+Cellular)',
        'Watch7,4': 'Apple Watch Series 9 45mm case (GPS+Cellular)',
        'Watch7,5': 'Apple Watch Ultra 2',
        'Watch7,8': 'Apple Watch Series 10 42mm case (GPS)',
        'Watch7,9': 'Apple Watch Series 10 46mm case (GPS)',
        'Watch7,10': 'Apple Watch Series 10 42mm case (GPS+Cellular)',
        'Watch7,11': 'Apple Watch Series 10 46mm (GPS+Cellular)',
    }


def configure_proxy(request, project, ci=None):
    """Configure HTTPS Proxy."""
    proxy_port = settings.PROXY_PORT
    logger.info('Starting HTTPS Proxy on %s', proxy_port)
    stop_httptools(get_http_tools_url(request))
    start_proxy(proxy_port, project)
    if ci:
        # Remote Port forward for HTTPS Proxy
        logger.info('Starting Remote Port Forward over SSH')
        Thread(target=ssh_jumphost_reverse_port_forward,
               args=(ci.get_ssh_connection_string(),),
               daemon=True).start()



import glob
import io
import itertools
import json
import logging
import os
import re
import stat
import subprocess
from collections import namedtuple

from django.conf import settings

from tldextract import extract

from mobsf.MobSF.utils import (
    append_scan_status,
    find_java_binary,
    is_file_exists,
    is_internet_available,
    update_local_db,
)

logger = logging.getLogger(__name__)


class Trackers:
    def __init__(self, checksum, apk_dir, tools_dir):
        self.checksum = checksum
        self.apk = None
        self.apk_dir = apk_dir
        self.tracker_db = os.path.join(
            settings.SIGNATURE_DIR,
            'exodus_trackers')
        self.signatures = None
        self.nb_trackers_signature = 0
        self.compiled_tracker_signature = None
        self.compiled_network_tracker_sig = None
        self.classes = None
        self.tools_dir = tools_dir
        self._update_tracker_db()

    def _update_tracker_db(self):
        """Update Trackers DB."""
        try:
            if not is_internet_available():
                logger.warning('No Internet Connection. '
                               'Skipping Trackers Database Update.')
                return
            exodus_db = '{}/api/trackers'.format(settings.EXODUS_URL)
            resp = update_local_db('Trackers',
                                   exodus_db,
                                   self.tracker_db)
            # Check1: SHA256 Change
            if resp:
                # DB needs update
                # Check2: DB Syntax Changed
                data = json.loads(resp.decode('utf-8', 'ignore'))
                is_db_format_good = False
                if 'trackers' in data:
                    if '1' in data['trackers']:
                        if 'code_signature' in data['trackers']['1']:
                            is_db_format_good = True
                if is_db_format_good:
                    # DB Format is not changed. Let's update DB
                    msg = 'Updating Trackers Database....'
                    logger.info(msg)
                    append_scan_status(self.checksum, msg)
                    with open(self.tracker_db, 'wb') as wfp:
                        wfp.write(resp)
                else:
                    desc = (
                        'Trackers Database format from '
                        'reports.exodus-privacy.eu.org has changed.'
                        ' Database is not updated. '
                        'Please report to: https://github.com/MobSF/'
                        'Mobile-Security-Framework-MobSF/issues'
                    )
                    logger.info(desc)
                    append_scan_status(
                        self.checksum,
                        'Tracker Database format changed',
                        desc)
        except Exception as exp:
            msg = '[ERROR] Trackers DB Update'
            logger.exception(msg)
            append_scan_status(self.checksum, msg, repr(exp))

    def _compile_signatures(self):
        """
        Compile Signatures.

        Compiles the regex associated to each signature, in order to speed up
        the trackers detection.
        :return: A compiled list of signatures.
        """
        self.compiled_tracker_signature = []
        self.compiled_network_tracker_sig = []
        try:
            self.compiled_tracker_signature = [
                re.compile(track.code_signature)
                for track in self.signatures]
            self.compiled_network_tracker_sig = [
                re.compile(track.network_signature)
                for track in self.signatures]
        except TypeError:
            logger.exception('compiling tracker signature failed')

    def load_trackers_signatures(self):
        """
        Load trackers signatures from the official Exodus database.

        :return: a dictionary of signatures.
        """
        self.signatures = []
        with io.open(self.tracker_db,
                     mode='r',
                     encoding='utf8',
                     errors='ignore') as flip:
            data = json.loads(flip.read())
        for elm in data['trackers']:
            self.signatures.append(
                namedtuple('tracker',
                           data['trackers'][elm].keys())(
                               *data['trackers'][elm].values()))
        self._compile_signatures()
        self.nb_trackers_signature = len(self.signatures)

    def get_embedded_classes(self):
        """
        Get the list of Java classes from all DEX files.

        :return: list of Java classes
        """
        if self.classes is not None:
            return self.classes
        for dex_file in glob.iglob(os.path.join(self.apk_dir, '*.dex')):
            # Fix dex permissions, malware mark dex as non read/write able
            if not os.access(dex_file, os.W_OK):
                os.chmod(dex_file, stat.S_IWRITE)
            if not os.access(dex_file, os.R_OK):
                os.chmod(dex_file, stat.S_IREAD)
            if (len(settings.BACKSMALI_BINARY) > 0
                    and is_file_exists(settings.BACKSMALI_BINARY)):
                bs_path = settings.BACKSMALI_BINARY
            else:
                bs_path = os.path.join(self.tools_dir, 'baksmali-3.0.8-dev-fat.jar')
            args = [find_java_binary(), '-jar',
                    bs_path, 'list', 'classes', dex_file]
            try:
                classes = subprocess.check_output(
                    args, universal_newlines=True).splitlines()
                if self.classes is not None:
                    self.classes = self.classes + classes
                else:
                    self.classes = classes
            except Exception:
                pass
        return self.classes

    def detect_trackers_in_list(self, class_list, network=False):
        """
        Detect embedded trackers in the provided classes list/urls.

        :return: list of embedded trackers
        """
        if self.signatures is None:
            self.load_trackers_signatures()

        def _detect_tracker(sig, tracker, class_list):
            for clazz in class_list:
                if sig.search(clazz):
                    return tracker
            return None

        results = []
        if network:
            compiled = self.compiled_network_tracker_sig
            args = [
                (compiled[index], tracker, class_list)
                for (index, tracker) in enumerate(self.signatures) if
                len(tracker.network_signature) > 3]

        else:
            compiled = self.compiled_tracker_signature
            args = [
                (compiled[index], tracker, class_list)
                for (index, tracker) in enumerate(self.signatures) if
                len(tracker.code_signature) > 3]

        for res in itertools.starmap(_detect_tracker, args):
            if res:
                results.append(res)

        trackers = [t for t in results if t is not None]
        trackers = sorted(trackers, key=lambda trackers: trackers.name)
        return trackers

    def detect_trackers(self):
        """
        Detect embedded trackers.

        :return: list of embedded trackers
        """
        if self.signatures is None:
            self.load_trackers_signatures()
        eclasses = self.get_embedded_classes()
        if eclasses:
            return self.detect_trackers_in_list(eclasses)
        return []

    def detect_runtime_trackers(self, items, deps=False):
        """
        Detect runtime trackers.

        :return: list of embedded trackers
        """
        if self.signatures is None:
            self.load_trackers_signatures()
        if items and not deps:
            # Domains
            return self.detect_trackers_in_list(items, True)
        elif items and deps:
            # Runtime Dependencies
            return self.detect_trackers_in_list(items)
        return []

    def get_trackers(self):
        """Get Trackers."""
        msg = 'Detecting Trackers'
        logger.info(msg)
        append_scan_status(self.checksum, msg)
        trackers = self.detect_trackers()
        tracker_dict = {'detected_trackers': len(trackers),
                        'total_trackers': self.nb_trackers_signature,
                        'trackers': []}
        for trk in trackers:
            trk_url = '{}/trackers/{}'.format(settings.EXODUS_URL, trk.id)
            tracker_dict['trackers'].append({
                'name': trk.name,
                'categories': ', '.join(trk.categories),
                'url': trk_url,
            })
        return tracker_dict

    def get_trackers_domains_or_deps(self, domains, deps):
        """Get Trackers from Runtime Deps/Domains."""
        trackers = []
        tracker_dict = {
            'detected_trackers': 0,
            'total_trackers': 0,
            'trackers': []}
        msg = 'Detecting Trackers from Domains'
        logger.info(msg)
        append_scan_status(self.checksum, msg)
        # Extract Trackers from Domains
        x_domains = set()
        for d in domains:
            cps = extract(d)
            x_domains.add(f'{cps.domain}.{cps.suffix}')
        trackers = self.detect_runtime_trackers(x_domains)
        # Extract Trackers from Runtime dependencies
        if deps:
            msg = 'Detecting Trackers from Runtime dependencies'
            logger.info(msg)
            append_scan_status(self.checksum, msg)
            runtime = self.detect_runtime_trackers(deps, True)
            for i in runtime:
                if i not in trackers:
                    trackers.append(i)
        tracker_dict['detected_trackers'] = len(trackers)
        tracker_dict['total_trackers'] = self.nb_trackers_signature
        for trk in trackers:
            trk_url = '{}/trackers/{}'.format(settings.EXODUS_URL, trk.id)
            tracker_dict['trackers'].append({
                'name': trk.name,
                'categories': ', '.join(trk.categories),
                'url': trk_url,
            })
        return tracker_dict



# -*- coding: utf_8 -*-
"""Module for iOS App Plist Analysis."""

import logging
import os
from plistlib import (
    FMT_XML,
    dumps,
    load,
    loads,
)
from pathlib import Path
from re import sub

from openstep_parser import OpenStepDecoder

from mobsf.MobSF.utils import (
    append_scan_status,
    find_key_in_dict,
    is_file_exists,
)
from mobsf.StaticAnalyzer.views.ios.kb.permission_analysis import (
    check_permissions,
)
from mobsf.StaticAnalyzer.views.ios.app_transport_security import (
    check_transport_security,
)
from mobsf.StaticAnalyzer.views.common.shared_func import (
    is_secret_key,
)

logger = logging.getLogger(__name__)
SKIP_PATH = {'__MACOSX', 'Pods'}
HIGH = 'high'
WARNING = 'warning'
INFO = 'info'
SECURE = 'secure'


def get_bundle_id(pobj, src):
    """Get iOS Bundle ID from source.

    Look up in Info.plist, entitlements, pbxproj
    """
    possible_ids = set()
    skip_chars = ('$(', '${')

    # From old Info.plist
    bundle_id_og = pobj.get('CFBundleIdentifier', '')
    if (not any(tmpl in bundle_id_og for tmpl in skip_chars)
            and len(bundle_id_og) > 1):
        possible_ids.add(bundle_id_og)

    # Look in entitlements, only present in newer iOS source
    path = Path(src)
    for p in path.rglob('*.entitlements'):
        if any(x in p.resolve().as_posix() for x in SKIP_PATH):
            continue
        try:
            ent = loads(p.read_bytes())
            groups = ent.get('com.apple.security.application-groups')
            if not groups:
                continue
            for i in groups:
                t = i.replace('.group', '').replace('group.', '')
                possible_ids.add(t.strip())
        except Exception:
            logger.warning('Error in parsing .entitlements')

    # Look in project.pbxproj
    for p in path.rglob('*.pbxproj'):
        if any(x in p.resolve().as_posix() for x in SKIP_PATH):
            continue
        try:
            search = 'PRODUCT_BUNDLE_IDENTIFIER'
            parsed = OpenStepDecoder.ParseFromString(p.read_text())
            if not parsed:
                continue
            for i in find_key_in_dict(search, parsed):
                if i.startswith(skip_chars):
                    continue
                for spl in skip_chars:
                    tc = f'.{spl}'
                    if tc in i:
                        i = i.split(tc)[0]
                possible_ids.add(i)
        except Exception:
            logger.warning('Error in parsing .pbxproj')
    if possible_ids:
        possible_ids = filter(None, possible_ids)
        # Fuzzy logic: return the shortest bundle id string
        return min(possible_ids, key=len)
    return ''


def convert_bin_xml(bin_xml_file):
    """Convert Binary XML to Readable XML."""
    try:
        with open(bin_xml_file, 'rb') as fp:
            plist_obj = load(fp)

        # Serializing the plist object to a binary plist string
        data = dumps(plist_obj, fmt=FMT_XML)
        Path(bin_xml_file).write_bytes(data)
    except Exception:
        logger.warning('Failed to convert plist')


def plist_analysis(checksum, src, scan_type):
    """Plist Analysis."""
    try:
        msg = 'iOS Info.plist Analysis Started'
        logger.info(msg)
        append_scan_status(checksum, msg)
        plist_info = {
            'bin_name': '',
            'bin': '',
            'id': '',
            'version': '',
            'build': '',
            'sdk': '',
            'pltfm': '',
            'min': '',
            'plist_xml': '',
            'permissions': {},
            'inseccon': {},
            'bundle_name': '',
            'build_version_name': '',
            'bundle_url_types': [],
            'bundle_supported_platforms': [],
            'bundle_version_name': '',
        }
        plist_file = None
        plist_files = []
        if scan_type == 'zip':
            msg = 'Finding Info.plist in iOS Source'
            logger.info(msg)
            append_scan_status(checksum, msg)
            for dirpath, _dirnames, files in os.walk(src):
                for name in files:
                    if (not any(x in dirpath for x in SKIP_PATH)
                            and name.endswith('.plist')):
                        plist_files.append(os.path.join(dirpath, name))
                        if name == 'GoogleService-Info.plist':
                            continue
                        if name == 'Info.plist' or '-Info.plist' in name:
                            plist_file = os.path.join(dirpath, name)
        else:
            msg = 'Finding Info.plist in iOS Binary'
            logger.info(msg)
            append_scan_status(checksum, msg)
            dirs = os.listdir(src)
            dot_app_dir = ''
            for dir_ in dirs:
                if dir_.endswith('.app'):
                    dot_app_dir = dir_
                    break
            bin_dir = os.path.join(src, dot_app_dir)  # Full Dir/Payload/x.app
            plist_file = os.path.join(bin_dir, 'Info.plist')
            plist_files = [plist_file]
        # Skip Plist Analysis if there is no Info.plist
        if not plist_file or not is_file_exists(plist_file):
            logger.warning(
                'Cannot find Info.plist file. Skipping Plist Analysis.')
            return plist_info

        # Generic Plist Analysis
        plist_obj = {}
        with open(plist_file, 'rb') as fp:
            plist_obj = load(fp)
        try:
            pxml = dumps(plist_obj).decode('utf-8', 'ignore')
        except Exception:
            logger.error('Failed to dump plist XML')
            pxml = ''
        plist_info['plist_xml'] = pxml
        plist_info['bin_name'] = (plist_obj.get('CFBundleDisplayName', '')
                                  or plist_obj.get('CFBundleName', ''))
        if not plist_info['bin_name'] and scan_type == 'ipa':
            # For iOS IPA
            plist_info['bin_name'] = dot_app_dir.replace('.app', '')
        plist_info['bin'] = plist_obj.get('CFBundleExecutable', '')
        plist_info['id'] = get_bundle_id(plist_obj, src)
        plist_info['build'] = plist_obj.get('CFBundleVersion', '')
        plist_info['sdk'] = plist_obj.get('DTSDKName', '')
        plist_info['pltfm'] = plist_obj.get('DTPlatformVersion', '')
        plist_info['min'] = plist_obj.get('MinimumOSVersion', '')
        plist_info['bundle_name'] = plist_obj.get('CFBundleName', '')
        plist_info['bundle_version_name'] = plist_obj.get(
            'CFBundleShortVersionString', '')
        btype = plist_obj.get('CFBundleURLTypes', [])
        if btype and isinstance(btype, dict):
            # Fixes bugs like # 1885
            btype = [btype]
        plist_info['bundle_url_types'] = btype
        plist_info['bundle_supported_platforms'] = plist_obj.get(
            'CFBundleSupportedPlatforms', [])
        logger.info('Checking Permissions')
        logger.info('Checking for Insecure Connections')
        ats = []
        for plist_file_ in plist_files:
            plist_obj_ = {}
            with open(plist_file_, 'rb') as fp:
                plist_obj_ = load(fp)
            # Check for app-permissions
            plist_info['permissions'].update(check_permissions(plist_obj_))
            # Check for ats misconfigurations
            ats += check_transport_security(plist_obj_)
        plist_info['inseccon'] = {
            'ats_findings': ats,
            'ats_summary': get_summary(ats),
        }
        return plist_info
    except Exception as exp:
        msg = 'Reading from Info.plist'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))


def get_summary(ats):
    """Get ATS finding summary."""
    if len(ats) == 0:
        return {}
    summary = {HIGH: 0, WARNING: 0, INFO: 0, SECURE: 0}
    for i in ats:
        if i['severity'] == HIGH:
            summary[HIGH] += 1
        elif i['severity'] == WARNING:
            summary[WARNING] += 1
        elif i['severity'] == INFO:
            summary[INFO] += 1
        elif i['severity'] == SECURE:
            summary[SECURE] += 1
    return summary


def get_plist_secrets(checksum, app_dir):
    """Get possible hardcoded secrets from plist files."""
    msg = 'Searching for secrets in plist files'
    logger.info(msg)
    append_scan_status(checksum, msg)
    result_list = set()

    def _remove_tags(data):
        """Remove tags from input."""
        return sub('<[^<]+>', '', data).strip()

    for i in Path(app_dir).rglob('*.plist'):
        xml_string = i.read_text('utf-8', 'ignore')
        xml_list = xml_string.split('\n')
        for index, line in enumerate(xml_list):
            if '<key>' in line and is_secret_key(_remove_tags(line)):
                nxt = index + 1
                value = (
                    _remove_tags(
                        xml_list[nxt])if nxt < len(xml_list) else False)
                if value and ' ' not in value:
                    result_list.add(
                        f'{_remove_tags(line)} : {_remove_tags(value)}')
    return list(result_list)



MANIFEST_DESC = {
    'well_known_assetlinks': {
        'title': ('App Link assetlinks.json file not found'
                  '[android:name=%s][android:host=%s]'),
        'level': 'high',
        'description': ('App Link asset verification URL (%s) not found or '
                        'configured incorrectly. (Status Code: %s). '
                        'App Links allow users to redirect from a web '
                        'URL/email to the mobile app. If this file is '
                        'missing or incorrectly configured for the '
                        'App Link host/domain, a malicious app can '
                        'hijack such URLs. This may lead to phishing '
                        'attacks, leak sensitive data in the URI, such '
                        'as PII, OAuth tokens, magic link/password '
                        'reset tokens and more. You must verify the '
                        'App Link domain by hosting the assetlinks.json '
                        'file and enabling verification via '
                        '[android:autoVerify="true"] in the Activity '
                        'intent-filter.'),
        'name': ('App Link assetlinks.json file not found '
                 '[android:name=%s], [android:host=%s]'),
    },
    'clear_text_traffic': {
        'title': ('Clear text traffic is Enabled For App'
                  '[android:usesCleartextTraffic=true]'),
        'level': 'high',
        'description': ('The app intends to use cleartext network traffic,'
                        ' such as cleartext HTTP, FTP stacks, DownloadManager,'
                        ' and MediaPlayer. The default value for'
                        ' apps that target API level 27 or lower is "true". '
                        'Apps that target API level 28 or higher default to'
                        ' "false". The key reason for avoiding cleartext'
                        ' traffic is the lack of confidentiality, '
                        'authenticity, and protections against tampering; '
                        'a network attacker can eavesdrop on transmitted '
                        'data and also modify it without being detected.'),
        'name': ('Clear text traffic is Enabled For App '
                 '[android:usesCleartextTraffic=true]'),
    },
    'direct_boot_aware': {
        'title': 'App is direct-boot aware [android:directBootAware=true]',
        'level': 'info',
        'description': ('This app can run before the user unlocks the device. '
                        'If you\'re using a custom subclass of Application, '
                        'and if any component inside your application is '
                        'direct - boot aware, then your entire custom '
                        'application is considered to be direct - boot aware.'
                        'During Direct Boot, your application can only access'
                        ' the data that is stored'
                        ' in device protected storage.'),
        'name': 'App is direct-boot aware [android:directBootAware=true]',
    },
    'has_network_security': {
        'title': ('App has a Network Security Configuration'
                  '[android:networkSecurityConfig=%s]'),
        'level': 'info',
        'description': ('The Network Security Configuration feature lets apps'
                        ' customize their network security settings in a safe,'
                        ' declarative configuration file without modifying '
                        'app code. These settings can be configured for '
                        'specific domains and for a specific app. '),
        'name': ('App has a Network Security Configuration '
                 '[android:networkSecurityConfig=%s]'),
    },
    'vulnerable_os_version': {
        'title': ('App can be installed on a vulnerable '
                  'unpatched Android version %s, [minSdk=%s]'),
        'level': 'high',
        'description': ('This application can be installed on an older version'
                        ' of android that has multiple unfixed '
                        'vulnerabilities. These devices won\'t receive '
                        'reasonable security updates from Google. '
                        'Support an Android version => 10, API 29 '
                        'to receive reasonable security updates.'),
        'name': ('App can be installed on a vulnerable '
                 'unpatched Android version %s, [minSdk=%s]'),
    },
    'vulnerable_os_version2': {
        'title': ('App can be installed on a vulnerable Android version %s,'
                  ' minSdk=%s]'),
        'level': 'warning',
        'description': ('This application can be installed on an older version'
                        ' of android that has multiple vulnerabilities. '
                        'Support an Android version => 10, API 29 '
                        'to receive reasonable security updates.'),
        'name': ('App can be installed on a vulnerable Android version'
                 ' %s, [minSdk=%s]'),
    },
    'app_is_debuggable': {
        'title': 'Debug Enabled For App [android:debuggable=true]',
        'level': 'high',
        'description': ('Debugging was enabled on the app which makes it '
                        'easier for reverse engineers to hook a debugger to'
                        ' it. This allows dumping a stack trace and accessing'
                        ' debugging helper classes.'),
        'name': 'Debug Enabled For App [android:debuggable=true]',
    },
    'app_allowbackup': {
        'title': ('Application Data can be Backed up [android:allowBackup=true]'),
        'level': 'warning',
        'description': ('This flag allows anyone to backup your application'
                        ' data via adb. It allows users who have enabled USB'
                        ' debugging to copy application data off of the'
                        ' device.'),
        'name': 'Application Data can be Backed up [android:allowBackup=true]',
    },
    'allowbackup_not_set': {
        'title': ('Application Data can be Backed up [android:allowBackup]'
                  ' flag is missing.'),
        'level': 'warning',
        'description': ('The flag [android:allowBackup] should be set to false'
                        '. By default it is set to true and allows anyone to '
                        'backup your application data via adb. It allows users'
                        ' who have enabled USB debugging to copy application '
                        'data off of the device.'),
        'name': ('Application Data can be Backed up [android:allowBackup] flag'
                 ' is missing.'),
    },
    'app_in_test_mode': {
        'title': 'Application is in Test Mode [android:testOnly=true]',
        'level': 'high',
        'description': ('It may expose functionality or data outside of itself'
                        ' that would cause a security hole.'),
        'name': 'Application is in Test Mode [android:testOnly=true]',
    },
    'task_affinity_set': {
        'title': 'TaskAffinity is set for activity (%s)',
        'level': 'warning',
        'description': ('If taskAffinity is set, then other application'
                        ' could read the Intents sent to Activities '
                        'belonging to another task. Always use the default'
                        ' setting keeping the affinity as the package name'
                        ' in order to prevent sensitive information inside'
                        ' sent or received Intents from being read by '
                        'another application.'),
        'name': 'TaskAffinity is set for Activity (%s)',
    },
    'non_standard_launchmode': {
        'title': 'Launch Mode of activity (%s) is not standard.',
        'level': 'warning',
        'description': ('An Activity should not be having the launch mode'
                        ' attribute set to "singleTask/singleInstance" as '
                        'it becomes root Activity and it is possible for'
                        ' other applications to read the contents of the'
                        ' calling Intent. So it is required to use the'
                        ' "standard" launch mode attribute when sensitive'
                        ' information is included in an Intent.'),
        'name': 'Launch Mode of activity (%s) is not standard.',
    },
    'task_hijacking': {
        'title': ('Activity (%s) is vulnerable to Android '
                  'Task Hijacking/StrandHogg.'),
        'level': 'high',
        'description': ('An Activity should not be having the launch mode '
                        'attribute set to "singleTask". It is then '
                        'possible for other applications to place a '
                        'malicious activity on top of the activity stack '
                        'resulting in Task Hijacking/StrandHogg 1.0 '
                        'vulnerability. This makes the application an easy '
                        'target for phishing attacks. The vulnerability can '
                        'be remediated by setting the launch mode attribute '
                        'to "singleInstance" or by setting an empty '
                        'taskAffinity (taskAffinity="") attribute. You can '
                        'also update the target SDK version (%s) of the '
                        'app to 28 or higher to fix this issue at '
                        'platform level.'),
        'name': ('Activity (%s) is vulnerable to Android '
                 'Task Hijacking/StrandHogg.'),
    },
    'task_hijacking2': {
        'title': 'Activity (%s) is vulnerable to StrandHogg 2.0',
        'level': 'high',
        'description': ('Activity is found to be vulnerable to '
                        'StrandHogg 2.0 task hijacking vulnerability. '
                        'When vulnerable, it is possible for other '
                        'applications to place a malicious activity '
                        'on top of the activity stack of the vulnerable '
                        'application. This makes the application an easy '
                        'target for phishing attacks. The vulnerability can '
                        'be remediated by setting the launch mode attribute '
                        'to "singleInstance" and by setting an empty '
                        'taskAffinity (taskAffinity=""). You can also update '
                        'the target SDK version (%s) of the app '
                        'to 29 or higher to fix this issue at '
                        'platform level.'),
        'name': 'Activity (%s) is vulnerable to StrandHogg 2.0',
    },
    'improper_provider_permission': {
        'title': 'Improper Content Provider Permissions [%s]',
        'level': 'warning',
        'description': ('A content provider permission was set to allows'
                        ' access from any other app on the device. '
                        'Content providers may contain sensitive '
                        'information about an app and therefore '
                        'should not be shared.'),
        'name': 'Improper Content Provider Permissions',
    },
    'dialer_code_found': {
        'title': ('Dailer Code: %s Found'
                  ' [android:scheme="android_secret_code"]'),
        'level': 'warning',
        'description': ('A secret code was found in the manifest. These codes,'
                        ' when entered into the dialer grant access to hidden'
                        ' content that may contain sensitive information.'),
        'name': ('Dailer Code: %s Found.'
                 ' [android:scheme="android_secret_code"]'),
    },
    'sms_receiver_port_found': {
        'title': 'Data SMS Receiver Set on Port: %s Found [android:port]',
        'level': 'warning',
        'description': ('A binary SMS receiver is configured to listen on a'
                        ' port. Binary SMS messages sent to a device are '
                        'processed by the application in whichever way the'
                        ' developer choses. The data in this SMS should be'
                        ' properly validated by the application. Furthermore,'
                        ' the application should assume that the SMS being'
                        ' received is from an untrusted source.'),
        'name': 'Data SMS Receiver Set on Port: %s Found. [android:port]',
    },
    'high_intent_priority_found': {
        'title': 'High Intent Priority (%s) - {%s} Hit(s) [android:priority]',
        'level': 'warning',
        'description': ('By setting an intent priority higher than another'
                        ' intent, the app effectively overrides '
                        'other requests.'),
        'name': 'High Intent Priority (%s) - {%s} Hit(s) [android:priority]',
    },
    'high_action_priority_found': {
        'title': 'High Action Priority (%s)[android:priority] ',
        'level': 'warning',
        'description': ('By setting an action priority higher than'
                        ' another action, the app effectively '
                        'overrides other requests.'),
        'name': 'High Action Priority (%s). [android:priority]',
    },
    'exported_protected_permission_signature': {
        'title': ('%s (%s) is Protected by a permission.'
                  '%s [android:exported=true]'),
        'level': 'info',
        'description': ('A%s %s is found to be exported, but'
                        ' is protected by permission.'),
        'name': ('%s %s is Protected by a permission.'
                 ' [%s] [android:exported=true]'),
    },
    'exported_protected_permission_normal': {
        'title': ('%s (%s) is Protected by a permission, but'
                  ' the protection level of the permission should be checked.'
                  '%s [android:exported=true]'),
        'level': 'warning',
        'description': ('A%s %s is found to be shared with other apps on the'
                        ' device therefore leaving it accessible to any other'
                        ' application on the device. It is protected by a '
                        'permission. However, the protection level of the'
                        '  permission is set to normal. This means that a '
                        'malicious application can request and obtain'
                        ' the permission and interact with the component.'
                        ' If it was set to signature, only applications '
                        'signed with the same certificate could obtain '
                        'the permission.'),
        'name': ('%s %s is Protected by a permission,'
                 ' but the protection level of'
                 ' the permission should be checked.'
                 ' [%s] [android:exported=true]'),
    },
    'exported_protected_permission_dangerous': {
        'title': ('%s (%s) is Protected by a permission, but'
                  ' the protection level of the permission should be checked.'
                  ' %s [android:exported=true]'),
        'level': 'warning',
        'description': ('A%s %s is found to be shared with other apps on the'
                        ' device therefore leaving it accessible to any other'
                        ' application on the device. It is protected by a'
                        ' permission. However, the protection level of the'
                        ' permission is set to dangerous. This means that a'
                        ' malicious application can request and obtain the'
                        ' permission and interact with the component. If it'
                        ' was set to signature, only applications signed with'
                        ' the same certificate could obtain the permission.'),
        'name': ('%s %s is Protected by a permission,'
                 ' but the protection level of'
                 ' the permission should be checked.'
                 ' [%s] [android:exported=true]'),
    },
    'exported_protected_permission_signatureorsystem': {
        'title': ('%s (%s) is Protected by a permission, but'
                  ' the protection level of the permission should be checked.'
                  ' %s [android:exported=true]'),
        'level': 'info',
        'description': ('A%s %s is found to be exported, but is protected by'
                        ' a permission. However, the protection level of the'
                        ' permission is set to signatureOrSystem. It is '
                        'recommended that signature level is used instead. '
                        'Signature level should suffice for most purposes, '
                        'and does not depend on where the applications are '
                        'installed on the device.'),
        'name': ('%s %s is Protected by a permission,'
                 ' but the protection level of'
                 ' the permission should be checked.'
                 ' [%s] [android:exported=true]'),
    },
    'exported_protected_permission_not_defined': {
        'title': ('%s (%s) is Protected by a permission, but'
                  ' the protection level of the permission should be checked.'
                  ' %s [android:exported=true]'),
        'level': 'warning',
        'description': ('A%s %s is found to be shared with other apps on the'
                        ' device therefore leaving it accessible to any other'
                        ' application on the device. It is protected by a '
                        'permission which is not defined in the analysed '
                        'application. As a result, the protection level of the'
                        ' permission should be checked where it is defined. If'
                        ' it is set to normal or dangerous, a malicious '
                        'application can request and obtain the permission and'
                        ' interact with the component. If it is set to '
                        'signature, only applications signed with the same '
                        'certificate can obtain the permission.'),
        'name': ('%s %s is Protected by a permission,'
                 ' but the protection level '
                 'of the permission should be '
                 'checked. [%s] [android:exported=true]'),
    },
    'exported_protected_permission_normal_app_level': {
        'title': ('%s (%s) is Protected by a permission at'
                  ' the application level, but the protection level of the '
                  'permission should be checked. %s [android:exported=true]'),
        'level': 'warning',
        'description': ('A%s %s is found to be shared with other apps on the'
                        ' device therefore leaving it accessible to any other'
                        ' application on the device.  It is protected by a '
                        'permission at the application level. However, the'
                        ' protection level of the permission is set to normal.'
                        ' This means that a malicious application can request '
                        'and obtain the permission and interact with the '
                        'component. If it was set to signature, only '
                        'applications signed with the same certificate '
                        'could obtain the permission.'),
        'name': ('%s %s is Protected by a permission at the application level,'
                 ' but the protection level of the permission should be '
                 'checked. [%s] [android:exported=true]'),
    },
    'exported_protected_permission_dangerous_app_level': {
        'title': ('%s (%s) is Protected by a permission at'
                  ' the application level, but the protection level of the '
                  'permission should be checked.'
                  ' %s [android:exported=true]'),
        'level': 'warning',
        'description': ('A%s %s is found to be shared with other apps on the'
                        ' device therefore leaving it accessible to any other'
                        ' application on the device. It is protected by a '
                        'permission at the application level. However, the '
                        'protection level of the permission is set to '
                        'dangerous. This means that a malicious application '
                        'can request and obtain the permission and interact '
                        'with the component. If it was set to signature, '
                        'only applications signed with the same certificate'
                        ' could obtain the permission.'),
        'name': ('%s %s is Protected by a permission'
                 ' at the application level, but'
                 ' the protection level of the permission should be '
                 'checked. [%s] [android:exported=true]'),
    },
    'exported_protected_permission': {
        'title': ('%s (%s)  Protected by a permission at'
                  ' the application level. %s [android:exported=true]'),
        'level': 'info',
        'description': ('A%s %s is found to be exported, but is protected by'
                        ' a permission at the application level.'),
        'name': ('%s %s Protected by a permission at the application level.'
                 ' [%s] [android:exported=true]'),
    },
    'exported_protected_permission_signatureorsystem_app_level': {
        'title': ('%s (%s) is Protected by a permission at '
                  'the application level, but the protection level of the '
                  'permission should be checked.'
                  ' %s [android:exported=true]'),
        'level': 'info',
        'description': ('A%s %s is found to be exported, but is protected '
                        'by a permission at the application level. However,'
                        ' the protection level of the permission is set to'
                        ' signatureOrSystem. It is recommended that '
                        'signature level is used instead. Signature level'
                        ' should suffice for most purposes, and does not '
                        'depend on where the applications are installed'
                        ' on the device.'),
        'name': ('%s %s is Protected by a permission'
                 ' at the application level, but'
                 ' the protection level of the permission should be checked.'
                 ' [%s] [android:exported=true]'),
    },
    'exported_protected_permission_app_level': {
        'title': ('%s (%s) is Protected by a permission'
                  ' at the application, but the protection level of the '
                  'permission should be checked.'
                  ' %s [android:exported=true]'),
        'level': 'warning',
        'description': ('A%s %s is found to be shared with other apps on '
                        'the device therefore leaving it accessible to any'
                        ' other application on the device. It is protected'
                        ' by a permission at the application level which is'
                        ' not defined in the analysed application. As a'
                        ' result, the protection level of the permission'
                        ' should be checked where it is defined. If it is'
                        ' set to normal or dangerous, a malicious application'
                        ' can request and obtain the permission and interact'
                        ' with the component. If it is set to signature, only'
                        ' applications signed with the same certificate can'
                        ' obtain the permission.'),
        'name': ('%s %s is Protected by a permission'
                 ' at the application, but the'
                 ' protection level of the permission should be checked.'
                 ' [%s] [android:exported=true]'),
    },
    'explicitly_exported': {
        'title': ('%s (%s) is not Protected.'
                  ' [android:exported=true]'),
        'level': 'warning',
        'description': ('A%s %s is found to be shared with other apps on the'
                        ' device therefore leaving it accessible to any other'
                        ' application on the device.'),
        'name': '%s %s is not Protected. [android:exported=true]',
    },
    'exported_intent_filter_exists': {
        'title': ('%s (%s) is not Protected.'
                  ' An intent-filter exists.'),
        'level': 'warning',
        'description': ('A%s %s is found to be shared with other apps on the'
                        ' device therefore leaving it accessible to any other '
                        'application on the device. The presence of '
                        'intent-filter indicates that the %s'
                        ' is explicitly exported.'),
        'name': '%s %s is not Protected.An intent-filter exists.',
    },
    'exported_provider': {
        'title': ('%s (%s) is not Protected. '
                  '[Content Provider, targetSdkVersion < 17]'),
        'level': 'warning',
        'description': ('A%s %s is found to be shared with other apps'
                        ' on the device therefore leaving it accessible '
                        'to any other application on the device. It is '
                        'a Content Provider that targets an API level '
                        'under 17, which makes it exported by default,'
                        ' regardless of the API level of the system '
                        'that the application runs on.'),
        'name': ('%s %s is not Protected.'
                 ' [Content Provider, targetSdkVersion < 17]'),
    },
    'exported_provider_2': {
        'title': ('%s (%s) would not be Protected if the'
                  ' application ran on a device where the the API level was'
                  ' less than 17. [Content Provider, '
                  'targetSdkVersion >= 17]'),
        'level': 'warning',
        'description': ('The Content Provider(%s %s) would be exported if the'
                        ' application ran on a device where the the API level '
                        'was less than 17. In that situation, it would be '
                        'shared with other apps on the device therefore '
                        'leaving it accessible to any other application '
                        'on the device.'),
        'name': ('%s %s would not be Protected if'
                 ' the application ran on a device'
                 ' where the the API level was less than 17.'
                 ' [Content Provider, targetSdkVersion >= 17]'),
    },
    'exported_provider_normal': {
        'title': ('%s (%s) is Protected by a permission, but'
                  ' the protection level of the permission should be checked.'
                  ' %s [Content Provider, targetSdkVersion < 17]'),
        'level': 'warning',
        'description': ('A%s %s is found to be shared with other apps on the'
                        ' device therefore leaving it accessible to any other'
                        ' application on the device. It is protected by a '
                        'permission. However, the protection level of the'
                        ' permission is set to normal. This means that a '
                        'malicious application can request and obtain '
                        'the permission and interact with the component. '
                        'If it was set to signature, only applications signed '
                        'with the same certificate could obtain '
                        'the permission.'),
        'name': ('%s %s is Protected by a permission,'
                 ' but the protection level'
                 ' of the permission should be checked.'
                 ' [%s] [Content Provider,'
                 ' targetSdkVersion < 17]'),
    },
    'exported_provider_danger': {
        'title': ('%s (%s) is Protected by a permission, '
                  'but the protection level of the permission should be '
                  'checked. %s [Content Provider, '
                  'targetSdkVersion < 17]'),
        'level': 'warning',
        'description': ('A%s %s is found to be shared with other apps on the'
                        ' device therefore leaving it accessible to any other'
                        ' application on the device. It is protected by a '
                        'permission. However, the protection level of the '
                        'permission is set to dangerous. This means that a '
                        'malicious application can request and obtain the '
                        'permission and interact with the component. If it'
                        ' was set to signature, only applications signed with'
                        ' the same certificate could obtain '
                        'the permission.'),
        'name': ('%s %s is Protected by a permission,'
                 ' but the protection level of '
                 'the permission should be checked. [%s] [Content Provider, '
                 'targetSdkVersion < 17]'),
    },
    'exported_provider_signature': {
        'title': ('%s (%s) is Protected by a permission.'
                  ' %s [Content Provider, targetSdkVersion < 17]'),
        'level': 'info',
        'description': ('A%s %s is found to be shared with other apps on the'
                        ' device therefore leaving it accessible to any other'
                        ' application on the device. It is '
                        'protected by permission.'),
        'name': ('%s %s is Protected by a permission. [%s] [Content Provider, '
                 'targetSdkVersion < 17]'),
    },
    'exported_provider_signatureorsystem': {
        'title': ('%s (%s) is Protected by a permission,'
                  ' but the protection level of the permission should be '
                  'checked. %s [Content Provider, '
                  'targetSdkVersion < 17]'),
        'level': 'info',
        'description': ('A%s %s is found to be exported, but is protected by'
                        ' a permission. However, the protection level of the'
                        ' permission is set to signatureOrSystem. It is'
                        ' recommended that signature level is used instead.'
                        ' Signature level should suffice for most purposes,'
                        ' and does not depend on where the applications are'
                        ' installed on the device.'),
        'name': ('%s %s is Protected by a permission,'
                 ' but the protection level of '
                 'the permission should be checked. [%s] [Content Provider, '
                 'targetSdkVersion < 17]'),
    },
    'exported_provider_unknown': {
        'title': ('%s (%s) is Protected by a permission, but'
                  ' the protection level of the permission should be checked.'
                  ' %s [Content Provider, targetSdkVersion < 17]'),
        'level': 'warning',
        'description': ('A%s %s is found to be shared with other apps on the '
                        'device therefore leaving it accessible to any other '
                        'application on the device. It is protected by a '
                        'permission which is not defined in the analysed '
                        'application. As a result, the protection level of the'
                        ' permission should be checked where it is defined. If'
                        ' it is set to normal or dangerous, a malicious '
                        'application can request and obtain the permission and'
                        ' interact with the component. If it is set to '
                        'signature, only applications signed with the same '
                        'certificate can obtain the permission.'),
        'name': ('%s %s is Protected by a permission,'
                 ' but the protection level of'
                 ' the permission should be checked. [%s] [Content Provider,'
                 ' targetSdkVersion < 17]'),
    },
    'exported_provider_normal_app': {
        'title': ('%s (%s) is Protected by a permission at'
                  ' the application level, but the protection level of the'
                  ' permission should be checked.'
                  ' %s [Content Provider, targetSdkVersion < 17]'),
        'level': 'warning',
        'description': ('A%s %s is found to be shared with other apps on the'
                        ' device therefore leaving it accessible to any other'
                        ' application on the device. It is protected by a'
                        ' permission at the application level. However, the'
                        ' protection level of the permission is set to normal.'
                        ' This means that a malicious application can request'
                        ' and obtain the permission and interact with the'
                        ' component. If it was set to signature, only '
                        'applications signed with the same certificate could'
                        ' obtain the permission.'),
        'name': ('%s %s is Protected by a permission'
                 ' at the application level, but'
                 ' the protection level of the permission should be checked.'
                 ' [%s] [Content Provider, targetSdkVersion < 17]'),
    },
    'exported_provider_danger_appl': {
        'title': ('%s (%s) is Protected by a permission at '
                  'the application level, but the protection level of the '
                  'permission should be checked.'
                  ' %s [Content Provider, targetSdkVersion < 17]'),
        'level': 'warning',
        'description': ('A%s %s is found to be shared with other apps on the'
                        ' device therefore leaving it accessible to any other'
                        ' application on the device. It is protected by a '
                        'permission at the application level. However, the '
                        'protection level of the permission is set to '
                        'dangerous. This means that a malicious application'
                        ' can request and obtain the permission and interact'
                        ' with the component. If it was set to signature, '
                        'only applications signed with the same certificate'
                        ' could obtain the permission.'),
        'name': ('%s %s is Protected by a permission'
                 ' at the application level, but'
                 ' the protection level of the permission should be checked.'
                 '[%s] [Content Provider, targetSdkVersion < 17]'),
    },
    'exported_provider_signature_appl': {
        'title': ('%s (%s) is Protected by a permission at'
                  ' the application level. %s [Content Provider,'
                  ' targetSdkVersion < 17]'),
        'level': 'info',
        'description': ('A%s %s is found to be shared with other apps on'
                        ' the device therefore leaving it accessible to any'
                        ' other application on the device. It is protected '
                        'by permission at the application level.'),
        'name': ('%s %s is Protected by a permission at the application level.'
                 '[%s] [Content Provider, targetSdkVersion < 17]'),
    },
    'exported_provider_signatureorsystem_app': {
        'title': ('%s (%s) is Protected by a permission at'
                  ' the application level, but the protection level of the'
                  ' permission should be checked.'
                  ' %s [Content Provider, targetSdkVersion < 17]'),
        'level': 'info',
        'description': ('A%s %s is found to be exported, but is protected by'
                        ' a permission at the application level. However, the'
                        ' protection level of the permission is set to '
                        'signatureOrSystem. It is recommended that signature '
                        'level is used instead. Signature level should suffice'
                        ' for most purposes, and does not depend on where the'
                        ' applications are installed on the device.'),
        'name': ('%s %s is Protected by a permission'
                 ' at the application level, '
                 'but the protection level of the permission should be '
                 'checked. [%s] [Content Provider, targetSdkVersion < 17]'),
    },
    'exported_provider_unknown_app': {
        'title': ('%s (%s) is Protected by a permission at'
                  ' application level, but the protection level of the '
                  'permission should be checked. %s '
                  '[Content Provider, targetSdkVersion < 17]'),
        'level': 'warning',
        'description': ('A%s %s is found to be shared with other apps on the'
                        ' device therefore leaving it accessible to any other'
                        ' application on the device. It is protected by a '
                        'permission at application level which is not defined'
                        ' in the analysed application. As a result, the '
                        'protection level of the permission should be checked'
                        ' where it is defined. If it is set to normal or '
                        'dangerous, a malicious application can request and'
                        ' obtain the permission and interact with the '
                        'component. If it is set to signature, only '
                        'applications signed with the same certificate '
                        'can obtain the permission.'),
        'name': ('%s %s is Protected by a permission at application level, but'
                 ' the protection level of the permission should be checked.'
                 ' [%s] [Content Provider, targetSdkVersion < 17]'),
    },
    'exported_provider_normal_new': {
        'title': ('%s (%s) is Protected by a permission, '
                  'but the protection level of the permission should be '
                  'checked if the application runs on a device where the '
                  'the API level is less than 17'
                  ' %s [Content Provider, targetSdkVersion >= 17]'),
        'level': 'warning',
        'description': ('The Content Provider (%s) would be exported if the'
                        ' application ran on a device where the the API level'
                        ' was less than 17. In that situation, it would still'
                        ' be protected by a permission. However, the '
                        'protection level of the permission is set to normal. '
                        'This means that a malicious application could request'
                        ' and obtain the permission and interact with the'
                        ' component. If it was set to signature, only'
                        ' applications signed with the same certificate '
                        'could obtain the permission.'),
        'name': ('%s %s is Protected by a permission,'
                 ' but the protection level of'
                 ' the permission should be checked if the application runs '
                 'on a device where the the API level is less than 17 '
                 ' [%s] [Content Provider, targetSdkVersion >= 17]'),
    },
    'exported_provider_danger_new': {
        'title': ('%s (%s) is Protected by a permission,'
                  ' but the protection level of the permission should be '
                  'checked if the application runs on a device where '
                  'the API level is less than 17. %s '
                  '[Content Provider, targetSdkVersion >= 17]'),
        'level': 'warning',
        'description': ('The Content Provider(%s) would be exported if the'
                        ' application ran on a device where the the API level'
                        ' was less than 17. In that situation, it would still'
                        ' be protected by a permission. However, the '
                        'protection level of the permission is set to'
                        ' dangerous. This means that a malicious application'
                        ' could request and obtain the permission and interact'
                        ' with the component. If it was set to signature, only'
                        ' applications signed with the same certificate could'
                        ' obtain the permission.'),
        'name': ('%s %s is Protected by a permission'
                 ', but the protection level of'
                 ' the permission should be checked if the application runs on'
                 ' a device where the the API level is less than 17.'
                 ' [%s] [Content Provider, targetSdkVersion >= 17]'),
    },
    'exported_provider_signature_new': {
        'title': ('%s (%s) is Protected by a permission.'
                  ' %s [Content Provider, targetSdkVersion >= 17]'),
        'level': 'info',
        'description': ('The Content Provider(%s) would be exported if the'
                        ' application ran on a device where the the API level'
                        ' was less than 17. Nevertheless, it is protected '
                        'by a permission.'),
        'name': ('%s %s is Protected by a permission.'
                 ' [%s] [Content Provider, targetSdkVersion >= 17]'),
    },
    'exported_provider_signatureorsystem_new': {
        'title': ('%s (%s) is Protected by a permission, but'
                  ' the protection level of the permission should be checked.'
                  ' %s [Content Provider, targetSdkVersion >= 17]'),
        'level': 'info',
        'description': ('The Content Provider(%s) would be exported if the '
                        'application ran on a device where the API level was'
                        ' less than 17. In that situation, it would still '
                        'be protected by a permission. However, the protection'
                        ' level of the permission is set to signatureOrSystem.'
                        ' It is recommended that signature level is used '
                        'instead. Signature level should suffice for most'
                        ' purposes, and does not depend on where the'
                        ' applications are installed on the device.'),
        'name': ('%s %s is Protected by a permission,'
                 ' but the protection level of'
                 ' the permission should be checked.'
                 ' [%s] [Content Provider, targetSdkVersion >= 17]'),
    },
    'exported_provider_unknown_new': {
        'title': ('%s (%s) is Protected by a permission, but'
                  ' the protection level of the permission should be checked'
                  ' if the application runs on a device where the the API '
                  'level is less than 17. %s '
                  '[Content Provider, targetSdkVersion >= 17]'),
        'level': 'warning',
        'description': ('The Content Provider(%s) would be exported if the'
                        ' application ran on a device where the the API level'
                        ' was less than 17. In that situation, it would still'
                        ' be protected by a permission which is not defined in'
                        ' the analysed application. As a result, the '
                        'protection level of the permission should be '
                        'checked where it is defined. If it is set to normal'
                        ' or dangerous, a malicious application can request'
                        ' and obtain the permission and interact with the '
                        'component. If it is set to signature, only '
                        'applications signed with the same certificate'
                        ' can obtain the permission.'),
        'name': ('%s %s is Protected by a permission, but the'
                 ' protection level of the permission should be'
                 ' checked if the application runs'
                 ' on a device where the the API level is less than 17.'
                 ' [%s] [Content Provider, targetSdkVersion >= 17]'),
    },
    'exported_provider_normal_app_new': {
        'title': ('%s (%s) is Protected by a permission at'
                  ' the application level, but the protection level of the'
                  ' permission should be checked if the application runs on'
                  ' a device where the the API level is less than 17'
                  ' %s [Content Provider, targetSdkVersion >= 17]'),
        'level': 'warning',
        'description': ('The Content Provider (%s) would be exported if the'
                        ' application ran on a device where the the API '
                        'level was less than 17. In that situation, it'
                        ' would still be protected by a permission. '
                        'However, the protection level of the permission'
                        ' is set to normal. This means that a malicious'
                        ' application could request and obtain the '
                        'permission and interact with the component. '
                        'If it was set to signature, only applications '
                        'signed with the same certificate could obtain'
                        ' the permission.'),
        'name': ('%s %s is Protected by a permission at the application level '
                 'should be checked, but the protection level of the '
                 'permission if the application runs on a device where'
                 ' the the API level is less than 17.'
                 ' [%s] [Content Provider, targetSdkVersion >= 17]'),
    },
    'exported_provider_danger_app_new': {
        'title': ('%s (%s) is Protected by a permission at'
                  ' the application level, but the protection level of the'
                  ' permission should be checked if the application runs on'
                  ' a device where the the API level is less than 17.'
                  ' %s [Content Provider, targetSdkVersion >= 17]'),
        'level': 'warning',
        'description': ('The Content Provider(%s) would be exported if the'
                        ' application ran on a device where the the API '
                        'level was less than 17. In that situation, it'
                        ' would still be protected by a permission. However,'
                        ' the protection level of the permission is set to'
                        ' dangerous. This means that a malicious application'
                        ' could request and obtain the permission and interact'
                        ' with the component. If it was set to signature, only'
                        ' applications signed with the same certificate could'
                        ' obtain the permission.'),
        'name': ('%s %s is Protected by a permission at the application'
                 ' level, but the protection level of the'
                 ' permission should be checked'
                 ' if the application runs on a device where the the API level'
                 ' is less than 17. [%s] '
                 '[Content Provider, targetSdkVersion >= 17]'),
    },
    'exported_provider_signature_app_new': {
        'title': ('%s (%s) is Protected by a permission at'
                  ' the application level. %s '
                  '[Content Provider, targetSdkVersion >= 17]'),
        'level': 'info',
        'description': ('The Content Provider(%s) would be exported if the'
                        ' application ran on a device where the the API '
                        'level was less than 17. Nevertheless, it is '
                        'protected by a permission.'),
        'name': ('%s %s is Protected by a permission at the application level.'
                 ' [%s] [Content Provider, targetSdkVersion >= 17]'),
    },
    'exported_provider_signatureorsystem_app_new': {
        'title': ('%s (%s) is Protected by a permission at'
                  ' the application level, but the protection level of the'
                  ' permission should be checked.'
                  ' %s [Content Provider, targetSdkVersion >= 17]'),
        'level': 'info',
        'description': ('The Content Provider(%s) would be exported if the'
                        ' application ran on a device where the API level was'
                        ' less than 17. In that situation, it would still be'
                        ' protected by a permission. However, the protection'
                        ' level of the permission is set to signatureOrSystem.'
                        ' It is recommended that signature level is used'
                        ' instead. Signature level should suffice for most'
                        ' purposes, and does not depend on where the '
                        'applications are installed on the device.'),
        'name': ('%s %s is Protected by a permission at the application'
                 ' level but the protection level of the permission'
                 ' should be checked.'
                 ' [%s] [Content Provider, targetSdkVersion >= 17]'),
    },
    'exported_provider_unknown_app_new': {
        'title': ('%s (%s) is Protected by a permission at '
                  'the application level, but the protection level of the '
                  'permission should be checked  if the application runs on'
                  ' a device where the the API level is less than 17.'
                  ' %s [Content Provider, targetSdkVersion >= 17]'),
        'level': 'warning',
        'description': ('The Content Provider(%s) would be exported if the'
                        ' application ran on a device where the the API level'
                        ' was less than 17. In that situation, it would still'
                        ' be protected by a permission which is not defined '
                        'in the analysed application. As a result, the'
                        ' protection level of the permission should be checked'
                        ' where it is defined. If it is set to normal or'
                        ' dangerous, a malicious application can request'
                        ' and obtain the permission and interact with the'
                        ' component. If it is set to signature, only '
                        'applications signed with the same certificate'
                        ' can obtain the permission.'),
        'name': ('%s %s is Protected by a permission at the application level,'
                 ' but the protection level of the permission should be'
                 ' checked  if the application runs on a device where the'
                 ' the API level is less than 17.'
                 ' [%s] [Content Provider, targetSdkVersion >= 17]'),
    },
}



# -*- coding: utf_8 -*-
"""SAST engine."""
import logging
import platform

from libsast import Scanner
from libsast.core_matcher.pattern_matcher import PatternMatcher
from libsast.core_matcher.choice_matcher import ChoiceMatcher
from libsast.common import get_worker_count

from django.conf import settings

from mobsf.MobSF.utils import (
    run_with_timeout,
)

logger = logging.getLogger(__name__)


def get_multiprocessing_strategy():
    """Get the multiprocessing strategy."""
    if settings.MULTIPROCESSING:
        # Settings take precedence
        mp = settings.MULTIPROCESSING
    elif platform.system() == 'Windows' and settings.ASYNC_ANALYSIS:
        # Set to thread on Windows for async analysis
        mp = 'thread'
    elif settings.ASYNC_ANALYSIS:
        # Set to billiard for async analysis
        mp = 'billiard'
    else:
        # Defaults to processpoolexecutor for sync analysis
        mp = 'default'
    return mp


class SastEngine:
    def __init__(self, options, path):
        self.root = path
        mp = get_multiprocessing_strategy()
        cpu_core = get_worker_count()
        options['cpu_core'] = cpu_core
        options['multiprocessing'] = mp
        if mp != 'default':
            logger.debug(
                'Multiprocessing strategy set to %s with (%d) CPU cores', mp, cpu_core)
        self.scan_paths = Scanner(options, [path]).get_scan_files()
        self.pattern_matcher = PatternMatcher(options)

    def scan(self):
        """Scan the files with given rules."""
        finds = run_with_timeout(
            self.pattern_matcher.scan,
            settings.SAST_TIMEOUT,
            self.scan_paths)
        return self.format_findings(finds)

    def read_files(self):
        """Read the files."""
        logger.info('Reading file contents for SAST')
        return self.pattern_matcher.read_file_contents(self.scan_paths)

    def run_rules(self, file_contents, rule_path):
        """Run the rules."""
        finds = run_with_timeout(
            self.pattern_matcher.regex_scan,
            settings.SAST_TIMEOUT,
            file_contents,
            rule_path)
        return self.format_findings(finds)

    def format_findings(self, findings):
        """Format the findings."""
        for details in findings.values():
            tmp_dict = {}
            for file_meta in details['files']:
                file_meta['file_path'] = file_meta[
                    'file_path'].replace(self.root, '', 1)
                file_path = file_meta['file_path']
                start = file_meta['match_lines'][0]
                end = file_meta['match_lines'][1]
                if start == end:
                    match_lines = start
                else:
                    exp_lines = []
                    for i in range(start, end + 1):
                        exp_lines.append(i)
                    match_lines = ','.join(str(m) for m in exp_lines)
                if file_path not in tmp_dict:
                    tmp_dict[file_path] = str(match_lines)
                elif tmp_dict[file_path].endswith(','):
                    tmp_dict[file_path] += str(match_lines)
                else:
                    tmp_dict[file_path] += ',' + str(match_lines)
            details['files'] = tmp_dict
        return findings


class ChoiceEngine:
    def __init__(self, options, path):
        self.root = path
        options['cpu_core'] = get_worker_count()
        options['multiprocessing'] = get_multiprocessing_strategy()
        self.scan_paths = Scanner(options, [path]).get_scan_files()
        self.choice_matcher = ChoiceMatcher(options)

    def read_files(self):
        """Read the files."""
        logger.info('Reading file contents for NIAP Scan')
        return self.choice_matcher.read_file_contents(self.scan_paths)

    def run_rules(self, file_contents, rule_path):
        """Run the rules."""
        return run_with_timeout(
            self.choice_matcher.regex_scan,
            settings.SAST_TIMEOUT,
            file_contents,
            rule_path)



# -*- coding: utf_8 -*-
"""Handle (.dylib) Dynamic library file."""
import logging
from pathlib import Path

import mobsf.MalwareAnalyzer.views.Trackers as Trackers
import mobsf.MalwareAnalyzer.views.VirusTotal as VirusTotal

from django.conf import settings
from django.shortcuts import render

from mobsf.MobSF.utils import (
    append_scan_status,
    file_size,
    print_n_send_error_response,
)
from mobsf.StaticAnalyzer.models import StaticAnalyzerIOS
from mobsf.StaticAnalyzer.views.ios.binary_analysis import (
    get_bin_info,
)
from mobsf.StaticAnalyzer.views.common.binary.lib_analysis import (
    library_analysis,
)
from mobsf.StaticAnalyzer.views.ios.binary_rule_matcher import (
    binary_rule_matcher,
)
from mobsf.StaticAnalyzer.views.ios.db_interaction import (
    get_context_from_db_entry,
    save_get_ctx,
)
from mobsf.StaticAnalyzer.views.ios.strings import (
    get_strings_metadata,
)
from mobsf.StaticAnalyzer.views.common.shared_func import (
    get_symbols,
    hash_gen,
)
from mobsf.StaticAnalyzer.views.common.firebase import (
    firebase_analysis,
)
from mobsf.MalwareAnalyzer.views.MalwareDomainCheck import (
    MalwareDomainCheck,
)
from mobsf.MobSF.views.authorization import (
    Permissions,
    has_permission,
)

logger = logging.getLogger(__name__)


def dylib_analysis(request, app_dict, rescan, api):
    """Independent Dylib (.dylib) analysis."""
    checksum = app_dict['md5_hash']
    app_dir = Path(app_dict['app_dir'])
    app_dict['app_file'] = f'{checksum}.dylib'
    app_dict['app_path'] = app_dir / app_dict['app_file']
    app_dict['app_path'] = app_dict['app_path'].as_posix()
    # DB
    ipa_db = StaticAnalyzerIOS.objects.filter(
        MD5=checksum)
    if ipa_db.exists() and not rescan:
        context = get_context_from_db_entry(ipa_db)
    else:
        if not has_permission(request, Permissions.SCAN, api):
            return print_n_send_error_response(
                request,
                'Permission Denied',
                False)
        append_scan_status(checksum, 'init')
        msg = 'Dylib Analysis Started'
        logger.info(msg)
        append_scan_status(checksum, msg)
        app_dict['size'] = str(
            file_size(app_dict['app_path'])) + 'MB'  # FILE SIZE
        app_dict['sha1'], app_dict['sha256'] = hash_gen(
            checksum,
            app_dict['app_path'])  # SHA1 & SHA256 HASHES
        app_dict['bin_dir'] = app_dict['app_dir']
        # Get Files
        all_files = {
            'files_short': [],
            'files_long': [],
            'special_files': [],
        }
        infoplist_dict = {
            'bin_name': '',
            'bin': '',
            'id': app_dict['file_name'],
            'version': '',
            'build': '',
            'sdk': '',
            'pltfm': '',
            'min': '',
            'plist_xml': '',
            'permissions': {},
            'inseccon': {},
            'bundle_name': '',
            'build_version_name': '',
            'bundle_url_types': [],
            'bundle_supported_platforms': [],
            'bundle_version_name': '',
        }
        app_dict['infoplist'] = infoplist_dict
        app_dict['all_files'] = all_files
        app_dict['appstore'] = ''
        app_dict['secrets'] = []
        bin_dict = {
            'checksec': {},
            'libraries': [],
            'bin_code_analysis': {},
            'strings': [],
            'bin_info': get_bin_info(
                Path(app_dict['app_path'])),
            'bin_type': 'Dylib',
        }
        # Analyze dylib
        dy = library_analysis(
            checksum,
            app_dict['bin_dir'],
            'macho')
        bin_dict['dylib_analysis'] = dy['macho_analysis']
        bin_dict['framework_analysis'] = {}
        # Store Symbols in File Analysis
        all_files['special_files'] = get_symbols(
            dy['macho_symbols'])
        # Binary code analysis on dylib symbols
        binary_rule_matcher(
            checksum,
            bin_dict['bin_code_analysis'],
            all_files['special_files'],
            b'')
        # Extract String metadata
        code_dict = get_strings_metadata(
            app_dict,
            bin_dict,
            all_files,
            dy['macho_strings'])
        # Domain Extraction and Malware Check
        code_dict['domains'] = MalwareDomainCheck().scan(
            checksum,
            code_dict['urls_list'])
        logger.info('Finished URL and Email Extraction')
        # Extract Trackers from Domains
        trk = Trackers.Trackers(
            checksum,
            None,
            app_dict['tools_dir'])
        trackers = trk.get_trackers_domains_or_deps(
            code_dict['domains'], [])
        code_dict['api'] = {}
        code_dict['code_anal'] = {}
        code_dict['firebase'] = firebase_analysis(
            checksum,
            code_dict)
        code_dict['trackers'] = trackers
        context = save_get_ctx(
            app_dict,
            code_dict,
            bin_dict,
            rescan)
    context['virus_total'] = None
    if settings.VT_ENABLED:
        vt = VirusTotal.VirusTotal(checksum)
        context['virus_total'] = vt.get_result(
            app_dict['app_path'])
    context['appsec'] = {}
    context['average_cvss'] = None
    template = 'static_analysis/ios_binary_analysis.html'
    if api:
        return context
    else:
        return render(request, template, context)



# -*- coding: utf_8 -*-
# Check against most common malware permissions.
import logging

from mobsf.MobSF.utils import (
    append_scan_status,
)


TOP_MALWARE_PERMISSIONS = [
    'android.permission.ACCEPT_HANDOVER',
    'android.permission.ACCESS_COARSE_LOCATION',
    'android.permission.ACCESS_FINE_LOCATION',
    'android.permission.ACCESS_NETWORK_STATE',
    'android.permission.CAMERA',
    'android.permission.GET_ACCOUNTS',
    'android.permission.GET_TASKS',
    'android.permission.INTERNET',
    'android.permission.SET_WALLPAPER',
    'android.permission.WRITE_SETTINGS',
    'android.permission.READ_SMS',
    'android.permission.SEND_SMS',
    'android.permission.RECEIVE_SMS',
    'android.permission.READ_CALL_LOG',
    'android.permission.READ_CONTACTS',
    'android.permission.RECORD_AUDIO',
    'android.permission.ACCESS_WIFI_STATE',
    'android.permission.READ_PHONE_STATE',
    'android.permission.RECEIVE_BOOT_COMPLETED',
    'android.permission.SYSTEM_ALERT_WINDOW',
    'android.permission.WAKE_LOCK',
    'android.permission.WRITE_EXTERNAL_STORAGE',
    'android.permission.READ_EXTERNAL_STORAGE',
    'android.permission.VIBRATE',
    'android.permission.REQUEST_INSTALL_PACKAGES',
]
OTHER_PERMISSIONS = [
    'android.permission.ACCESS_BACKGROUND_LOCATION',
    'android.permission.ACCESS_CHECKIN_PROPERTIES',
    'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS',
    'android.permission.ACCESS_MOCK_LOCATION',
    'android.permission.ACCESS_NOTIFICATION_POLICY',
    'android.permission.ACCESS_SUPERUSER',
    'android.permission.ACCESS_SURFACE_FLINGER',
    'android.permission.ACCOUNT_MANAGER',
    'android.permission.ACTIVITY_RECOGNITION',
    'android.permission.AUTHENTICATE_ACCOUNTS',
    'android.permission.BATTERY_STATS',
    'android.permission.BIND_APPWIDGET',
    'android.permission.BIND_DEVICE_ADMIN',
    'android.permission.BIND_INPUT_METHOD',
    'android.permission.BIND_REMOTEVIEWS',
    'android.permission.BIND_WALLPAPER',
    'android.permission.BLUETOOTH',
    'android.permission.BLUETOOTH_ADMIN',
    'android.permission.BRICK',
    'android.permission.BROADCAST_PACKAGE_REMOVED',
    'android.permission.BROADCAST_SMS',
    'android.permission.BROADCAST_STICKY',
    'android.permission.BROADCAST_WAP_PUSH',
    'android.permission.CALL_PHONE',
    'android.permission.CHANGE_NETWORK_STATE',
    'android.permission.CHANGE_WIFI_STATE',
    'android.permission.DIAGNOSTIC',
    'android.permission.FLASHLIGHT',
    'android.permission.FORCE_STOP_PACKAGES',
    'android.permission.FOREGROUND_SERVICE',
    'android.permission.GET_ACCOUNTS_PRIVELEGED',
    'android.permission.MODIFY_AUDIO_SETTINGS',
    'android.permission.MOUNT_FORMAT_FILESYSTEMS',
    'android.permission.PROCESS_OUTGOING_CALLS',
    'android.permission.READ_CALENDAR',
    'android.permission.PACKAGE_USAGE_STATS',
    'android.permission.REQUEST_IGNORE_BATTERY_OPTIMIZATIONS',
    'android.permission.WRITE_CONTACTS',
    'android.permission.WRITE_SMS',
    'com.android.launcher.permission.INSTALL_SHORTCUT',
    'com.google.android.c2dm.permission.RECEIVE',
    'com.google.android.finsky.permission.BIND_GET_INSTALL_REFERRER_SERVICE',
    'com.google.android.gms.permission.ACTIVITY_RECOGNITION',
    'com.google.android.gms.permission.AD_ID',
]


logger = logging.getLogger(__name__)


def check_malware_permission(checksum, perms):
    """Check against most common malware permissions."""
    msg = 'Checking for Malware Permissions'
    logger.info(msg)
    append_scan_status(checksum, msg)
    malware_perms = []
    other_perms = []
    for permission in perms:
        if permission in TOP_MALWARE_PERMISSIONS:
            malware_perms.append(permission)
        if permission in OTHER_PERMISSIONS:
            other_perms.append(permission)
    data = {
        'top_malware_permissions': malware_perms,
        'other_abused_permissions': other_perms,
        'total_malware_permissions': len(TOP_MALWARE_PERMISSIONS),
        'total_other_permissions': len(OTHER_PERMISSIONS),
    }
    return data



import logging
from pathlib import Path
from enum import Enum

from django.conf import settings

from mobsf.MalwareAnalyzer.views.MalwareDomainCheck import (
    MalwareDomainCheck,
)
from mobsf.StaticAnalyzer.views.common.shared_func import (
    url_n_email_extract,
)
from mobsf.StaticAnalyzer.views.sast_engine import SastEngine
from mobsf.MobSF.utils import (
    append_scan_status,
)

logger = logging.getLogger(__name__)


class _SourceType(Enum):
    swift = 'Swift'
    objc = 'Objective-C'
    swift_and_objc = 'Swift, Objective-C'
    nocode = 'No Code'


def merge_findings(swift, objc):
    code_analysis = {}
    # Add all unique keys
    for k in swift:
        if k in objc:
            swift[k]['files'].update(objc[k]['files'])
        code_analysis[k] = swift[k]
    for k in objc:
        if k not in code_analysis:
            code_analysis[k] = objc[k]
    return code_analysis


def ios_source_analysis(checksum, src):
    """IOS Objective-C and Swift Code Analysis."""
    try:
        logger.info('Starting iOS Source Code and PLIST Analysis')
        root = Path(settings.BASE_DIR) / 'StaticAnalyzer' / 'views'
        swift_rules = root / 'ios' / 'rules' / 'swift_rules.yaml'
        objective_c_rules = root / 'ios' / 'rules' / 'objective_c_rules.yaml'
        api_rules = root / 'ios' / 'rules' / 'ios_apis.yaml'
        code_findings = {}
        api_findings = {}
        email_n_file = []
        url_n_file = []
        url_list = []
        domains = {}
        source_type = ''
        source_types = set()
        skp = settings.SKIP_CLASS_PATH
        msg = 'iOS Source Code Analysis Started'
        logger.info(msg)
        append_scan_status(checksum, msg)

        # Objective C Analysis
        options = {
            'match_rules': objective_c_rules.as_posix(),
            'match_extensions': {'.m'},
            'ignore_paths': skp,
        }
        objc_findings = SastEngine(options, src).scan()
        if objc_findings:
            source_types.add(_SourceType.objc)

        # Swift Analysis
        options = {
            'match_rules': swift_rules.as_posix(),
            'match_extensions': {'.swift'},
            'ignore_paths': skp,
        }
        swift_findings = SastEngine(options, src).scan()
        if swift_findings:
            source_types.add(_SourceType.swift)
        code_findings = merge_findings(swift_findings, objc_findings)
        msg = 'iOS Source Code Analysis Completed'
        logger.info(msg)
        append_scan_status(checksum, msg)

        # API Analysis
        msg = 'iOS API Analysis Started'
        logger.info(msg)
        append_scan_status(checksum, msg)
        options = {
            'match_rules': api_rules.as_posix(),
            'match_extensions': {'.m', '.swift'},
            'ignore_paths': skp,
        }
        api_findings = SastEngine(options, src).scan()
        msg = 'iOS API Analysis Completed'
        logger.info(msg)
        append_scan_status(checksum, msg)

        # Extract URLs and Emails
        msg = 'Extracting Emails and URLs from Source Code'
        logger.info(msg)
        append_scan_status(checksum, msg)
        for pfile in Path(src).rglob('*'):
            if (
                (pfile.suffix in ('.m', '.swift')
                    and any(skip_path in pfile.as_posix()
                            for skip_path in skp) is False
                    and pfile.is_file())
            ):
                content = None
                try:
                    content = pfile.read_text('utf-8', 'ignore')
                    # Certain file path cannot be read in windows
                except Exception:
                    continue
                relative_src_path = pfile.as_posix().replace(src, '')
                urls, urls_nf, emails_nf = url_n_email_extract(
                    content, relative_src_path)
                url_list.extend(urls)
                url_n_file.extend(urls_nf)
                email_n_file.extend(emails_nf)
        msg = 'Email and URL Extraction Completed'
        logger.info(msg)
        append_scan_status(checksum, msg)
        if not source_types:
            source_type = _SourceType.nocode.value
        elif len(source_types) > 1:
            source_type = _SourceType.swift_and_objc.value
        else:
            source_type = source_types.pop().value

        urls_list = list(set(url_list))
        # Domain Extraction and Malware Check
        domains = MalwareDomainCheck().scan(
            checksum,
            urls_list)
        msg = 'Finished Code Analysis, Email and URL Extraction'
        logger.info(msg)
        append_scan_status(checksum, msg)
        code_analysis_dict = {
            'api': api_findings,
            'code_anal': code_findings,
            'urls_list': urls_list,
            'urlnfile': url_n_file,
            'domains': domains,
            'emailnfile': email_n_file,
            'source_type': source_type,
        }
        return code_analysis_dict
    except Exception as exp:
        msg = 'iOS Source Analysis Failed'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))



# -*- coding: utf_8 -*-
"""Use aapt2 to extract APK features."""
import re
import logging
import subprocess
from platform import system
from pathlib import Path

from django.conf import settings

from mobsf.MobSF.utils import (
    find_aapt,
)

logger = logging.getLogger(__name__)


class AndroidAAPT:

    def __init__(self, apk_path):
        self.aapt2_path = None
        self.aapt_path = None
        self.apk_path = apk_path
        self.data = {
            'permissions': [],
            'uses_features': {},
            'package': None,
            'application_label': None,
            'application_icon': None,
            'launchable_activity': None,
            'min_sdk_version': None,
            'target_sdk_version': None,
        }

        # Check for custom AAPT2 path in settings
        if (getattr(settings, 'AAPT2_BINARY', '')
                and len(settings.AAPT2_BINARY) > 0
                and Path(settings.AAPT2_BINARY).exists()):
            self.aapt2_path = settings.AAPT2_BINARY
        else:
            aapt2 = 'aapt2.exe' if system() == 'Windows' else 'aapt2'
            self.aapt2_path = find_aapt(aapt2)

        # Check for custom AAPT path in settings
        if (getattr(settings, 'AAPT_BINARY', '')
                and len(settings.AAPT_BINARY) > 0
                and Path(settings.AAPT_BINARY).exists()):
            self.aapt_path = settings.AAPT_BINARY
        else:
            aapt = 'aapt.exe' if system() == 'Windows' else 'aapt'
            self.aapt_path = find_aapt(aapt)

        # Ensure both aapt and aapt2 are found
        if not (self.aapt2_path and self.aapt_path):
            raise FileNotFoundError('aapt and aapt2 found')

    def _execute_command(self, args):
        try:
            out = subprocess.check_output(
                args,
                stderr=subprocess.STDOUT)
            return out.decode('utf-8', errors='ignore')
        except subprocess.CalledProcessError as e:
            logger.warning(e.output)
            return None

    def _get_strings(self, output):
        # Regex to match strings while ignoring paths (strings without slashes)
        pattern = r'String #[\d]+ : ([^\/\n]+)'
        matches = re.findall(pattern, output)
        # Strip whitespace and return the extracted strings
        return [match.strip() for match in matches]

    def _parse_badging(self, output):
        # Match the package information
        package_match = re.search(r'package: name=\'([\w\.]+)\'', output)
        if package_match:
            self.data['package'] = package_match.group(1)

        # Match permissions
        permissions = re.findall(r'uses-permission: name=\'([\w\.]+)\'', output)
        if permissions:
            self.data['permissions'] = permissions

        # Match minSdkVersion
        min_sdk_match = re.search(r'minSdkVersion:\'(\d+)\'', output)
        if min_sdk_match:
            self.data['min_sdk_version'] = min_sdk_match.group(1)

        # Match targetSdkVersion
        target_sdk_match = re.search(r'targetSdkVersion:\'(\d+)\'', output)
        if target_sdk_match:
            self.data['target_sdk_version'] = target_sdk_match.group(1)

        # Match application label
        label_match = re.search(r'application-label(?:-[\w\-]+)?:\'([^\']+)\'', output)
        if label_match:
            self.data['application_label'] = label_match.group(1)

        # Match application icon
        icon_match = re.search(r'application:.*icon=\'([^\']+)\'', output)
        if icon_match:
            self.data['application_icon'] = icon_match.group(1)

        # Match launchable activity
        activity_match = re.search(r'launchable-activity: name=\'([\w\.]+)\'', output)
        if activity_match:
            self.data['launchable_activity'] = activity_match.group(1)

        # Match used features
        features = {}
        feature_matches = re.findall(
            (r'(uses-feature(?:-not-required)?|uses-implied-feature): '
             r'name=\'([\w\.]+)\'(?: reason=\'([^\']+)\')?'),
            output,
        )
        for feature_type, feature_name, reason in feature_matches:
            features[feature_name] = {
                'type': feature_type,
                # e.g., 'uses-feature',
                # 'uses-feature-not-required',
                # 'uses-implied-feature'
                'reason': reason if reason else 'No reason provided',
            }
        self.data['uses_features'] = features

        return self.data

    def get_apk_files(self):
        """List all files in the APK."""
        output = self._execute_command(
            [self.aapt_path, 'list', self.apk_path])
        if output:
            return output.splitlines()
        return []

    def get_apk_strings(self):
        """Extract strings from the APK."""
        output = self._execute_command(
            [self.aapt2_path, 'dump', 'strings', self.apk_path])
        if output:
            return self._get_strings(output)
        return []

    def get_apk_features(self):
        """Extract features from the APK."""
        output = self._execute_command(
            [self.aapt2_path, 'dump', 'badging', self.apk_path])
        if output:
            return self._parse_badging(output)
        return self.data



import logging
import sys
import time
from pathlib import Path
from threading import Thread

import frida

from django.conf import settings

from mobsf.DynamicAnalyzer.views.common.frida import (
    get_bridge_loader,
)
from mobsf.DynamicAnalyzer.views.ios.corellium_ssh import (
    ssh_jumphost_port_forward,
)
from mobsf.DynamicAnalyzer.views.ios.frida_auxiliary_scripts import (
    class_pattern,
    class_trace,
    classes_with_method,
    get_loaded_classes,
    get_loaded_classes_methods,
    get_methods,
    string_capture,
    string_compare,
)

SCREENSHOT_CAPTURED_MSG = '✅ Screenshot Captured'


logger = logging.getLogger(__name__)
_PID = None


class Frida:

    def __init__(
            self,
            ssh_string,
            app_hash,
            bundle_id,
            defaults,
            dump,
            auxiliary,
            extras,
            code):
        self.ssh_connection_string = ssh_string
        self.app_container = None
        self.hash = app_hash
        self.bundle_id = bundle_id
        self.defaults = defaults
        self.dump = dump
        self.auxiliary = auxiliary
        self.extras = extras
        self.code = code
        self.frida_dir = Path(settings.TOOLS_DIR) / 'frida_scripts' / 'ios'
        self.ipa_dir = Path(settings.UPLD_DIR) / self.hash
        self.frida_log = self.ipa_dir / 'mobsf_frida_out.txt'
        self.dump_file = self.ipa_dir / 'mobsf_dump_file.txt'
        self.container_file = self.ipa_dir / 'mobsf_app_container_path.txt'
        self.not_supported_text = (
            'Failed to instrument the app with Frida. '
            'This app is not supported by Frida. '
            'Are you able to run the app on '
            'this device?')
        self.bridge_loader = get_bridge_loader()
        self.screenshot_event = None

    def get_scripts(self, script_type, selected_scripts):
        """Get Frida Scripts."""
        combined_script = []
        header = []
        if not selected_scripts:
            return header
        all_scripts = self.frida_dir / script_type
        for script in all_scripts.rglob('*.js'):
            if '*' in selected_scripts:
                combined_script.append(script.read_text('utf-8', 'ignore'))
            if script.stem in selected_scripts:
                header.append(f'send("Loaded Frida Script - {script.stem}");')
                combined_script.append(script.read_text('utf-8', 'ignore'))
        return header + combined_script

    def get_auxiliary(self):
        """Get auxiliary hooks."""
        scripts = []
        if not self.auxiliary:
            return scripts
        for itm in self.auxiliary:
            if itm == 'enum_class':
                scripts.append(get_loaded_classes())
            elif itm == 'enum_class_methods':
                scripts.append(get_loaded_classes_methods())
            elif itm == 'string_capture':
                scripts.append(string_capture())
            elif itm == 'string_compare':
                scripts.append(string_compare())
            elif itm == 'enum_methods' and 'class_name' in self.extras:
                scripts.append(
                    get_methods(self.extras['class_name']))
            elif itm == 'search_class' and 'class_search' in self.extras:
                scripts.append(
                    class_pattern(self.extras['class_search']))
            elif itm == 'search_method' and 'method_search' in self.extras:
                scripts.append(
                    classes_with_method(self.extras['method_search']))
            elif itm == 'trace_class' and 'class_trace' in self.extras:
                scripts.append(
                    class_trace(self.extras['class_trace']))
        return scripts

    def get_script(self, nolog=False):
        """Get final script."""
        if not self.code:
            self.code = ''
        rpc_list = []
        scripts = [self.code]
        scripts.extend(self.get_scripts('default', self.defaults))
        scripts.extend(self.get_scripts('dump', self.dump))
        rpc_list.extend(self.get_scripts('rpc', ['*']))
        scripts.extend(self.get_auxiliary())
        rpc_script = ','.join(rpc_list)
        rpc = f'rpc.exports = {{ \n{rpc_script}\n }};'
        combined = '\n'.join(scripts)
        final = f'{rpc}\n setTimeout(function() {{ \n{combined}\n }}, 1000)'

        # Use bridges only for Frida 17.0.0+
        if self.bridge_loader.is_required_and_available():
            try:
                final = self.bridge_loader.inject_bridge_support(
                    final, 'objc')
                if not nolog:
                    logger.info('Frida Script injected with bridges')
            except Exception:
                logger.exception(
                    'Failed to inject script with bridges '
                    '(required for Frida 17.0.0+)')
        else:
            if not nolog:
                logger.info('Frida Script injected')

        return final

    def frida_response(self, message, data):
        """Function to handle frida responses."""
        if 'payload' in message:
            msg = message['payload']
            aux = '[AUXILIARY] '
            jb = '[Jailbreak Detection Bypass] '
            dump = '[MBSFDUMP] '
            if not isinstance(msg, str):
                msg = str(msg)
            if dump in msg:
                msg = msg.replace(dump, '')
                self.write_log(self.dump_file, f'{msg}\n')
            elif msg.startswith(jb):
                self.write_log(self.frida_log, f'{msg}\n')
            elif msg.startswith(aux):
                msg = msg.replace(aux, '[*] ')
                self.write_log(self.frida_log, f'{msg}\n')
            else:
                logger.debug('[Frida] %s', msg)
                self.write_log(self.frida_log, f'{msg}\n')
                if (self.screenshot_event
                        and SCREENSHOT_CAPTURED_MSG in msg):
                    self.screenshot_event.set()
        else:
            logger.error('[Frida] %s', message)

    def frida_ssh_forward(self):
        """Setup SSH tunnel and port forwarding for corellium."""
        try:
            logger.info('Setting up SSH tunnel and port forwarding')
            # Corellium VM provides SSH over bastion host
            # Frida server is not reachable, open SSH tunnel
            # and port forward
            Thread(
                target=ssh_jumphost_port_forward,
                args=(self.ssh_connection_string,),
                daemon=True).start()
            time.sleep(3)
        except Exception:
            logger.exception('Setting up SSH tunnel')

    def spawn(self):
        """Connect to Frida Server and spawn the app."""
        global _PID
        try:
            try:
                self.clean_up()
                _PID = frida.get_remote_device().spawn([self.bundle_id])
            except frida.NotSupportedError:
                logger.error(self.not_supported_text)
                return
            except frida.ServerNotRunningError:
                self.frida_ssh_forward()
                _PID = None
            if not _PID:
                _PID = frida.get_remote_device().spawn([self.bundle_id])
            logger.info('Spawning %s', self.bundle_id)
            time.sleep(2)
        except frida.TimedOutError:
            logger.error('Timed out while waiting for device to appear')
        except frida.ServerNotRunningError:
            logger.error('Frida Server is not running')
        except frida.NotSupportedError:
            logger.error(self.not_supported_text)
            return
        except (frida.ProcessNotFoundError,
                frida.ProcessNotRespondingError,
                frida.TransportError,
                frida.InvalidOperationError):
            pass
        except Exception:
            logger.exception('Error Connecting to Frida Server')

    def session(self, pid, bundle_id):
        """Use existing session to inject frida scripts."""
        global _PID
        try:
            try:
                device = frida.get_remote_device()
                if pid and bundle_id:
                    _PID = pid
                    self.bundle_id = bundle_id
                try:
                    front = device.get_frontmost_application()
                    if front or front.pid != _PID:
                        # Not the front most app.
                        # Get the pid of the front most app
                        logger.warning('Front most app has PID %s', front.pid)
                        _PID = front.pid
                except Exception:
                    pass
                session = device.attach(_PID)
            except frida.NotSupportedError:
                logger.error(self.not_supported_text)
                return
            except Exception:
                logger.warning('Cannot attach to pid, spawning again.')
                self.spawn()
                session = device.attach(_PID)
            if session and device and _PID:
                script = session.create_script(self.get_script())
                script.on('message', self.frida_response)
                script.load()
                api = script.exports_sync
                device.resume(_PID)
                self.api_handler(api)
                sys.stdin.read()
                script.unload()
                session.detach()
        except frida.NotSupportedError:
            logger.error(self.not_supported_text)
        except (frida.ProcessNotFoundError,
                frida.ProcessNotRespondingError,
                frida.TransportError,
                frida.InvalidOperationError):
            pass
        except Exception:
            logger.exception('Error Connecting to Frida Server')

    def ps(self):
        """Get running process pid."""
        ps_dict = []
        try:
            try:
                device = frida.get_remote_device()
                processes = device.enumerate_applications(scope='minimal')
            except frida.ServerNotRunningError:
                self.frida_ssh_forward()
                device = frida.get_remote_device()
                processes = device.enumerate_applications(scope='minimal')
            if device and processes:
                for process in processes:
                    if process.pid != 0:
                        ps_dict.append({
                            'pid': process.pid,
                            'name': process.name,
                            'identifier': process.identifier,
                        })
        except Exception:
            logger.exception('Failed to enumerate running applications')
        return ps_dict

    def api_handler(self, api):
        """Call Frida rpc functions."""
        loaded_classes = []
        loaded_class_methods = []
        implementations = []
        try:
            try:
                self.app_container = api.get_container()
                self.container_file.write_text(self.app_container)
            except frida.InvalidOperationError:
                pass
            # Migrating defaults to rpc functions
            if 'ssl_pinning' in self.defaults:
                api.bypass_ios_ssl_pinning()

            if not self.extras:
                return
            screenshot_action = self.extras.get('screenshot_action')
            if screenshot_action == 'capture':
                api.get_screenshot()
            raction = self.extras.get('rclass_action')
            rclass = self.extras.get('rclass_name')
            rclass_pattern = self.extras.get('rclass_pattern')
            rmethod = self.extras.get('rmethod_name')
            rmethod_pattern = self.extras.get('rmethod_pattern')
            if raction == 'raction':
                loaded_classes = api.getLoadedClasses()
            elif raction == 'getclasses' and rclass_pattern:
                loaded_classes = api.getLoadedClasses(f'/{rclass_pattern}/i')
            elif raction == 'getmethods' and rclass and rmethod:
                loaded_class_methods = api.getMethods(rclass)
            elif raction == 'getmethods' and rclass and rmethod_pattern:
                loaded_class_methods = api.getMethods(
                    rclass,
                    f'/{rmethod_pattern}/i')
            elif raction == 'getimplementations' and rclass and rmethod:
                implementations = api.getImplementations(rclass, rmethod)
        except Exception:
            logger.exception('Error while calling Frida RPC functions')
        if loaded_classes:
            loaded_classes = sorted(loaded_classes)
            rpc_classes = self.ipa_dir / 'mobsf_rpc_classes.txt'
            rpc_classes.write_text('\n'.join(
                loaded_classes), 'utf-8')
        if loaded_class_methods:
            loaded_class_methods = sorted(loaded_class_methods)
            rpc_methods = self.ipa_dir / 'mobsf_rpc_methods.txt'
            rpc_methods.write_text('\n'.join(
                loaded_class_methods), 'utf-8')
        if implementations:
            implementations = sorted(implementations)
            rpc_impl = self.ipa_dir / 'mobsf_rpc_impl.txt'
            rpc_impl.write_text('\n'.join(
                implementations), 'utf-8')

    def clean_up(self):
        if self.frida_log.exists():
            self.frida_log.unlink()
        if self.dump_file.exists():
            self.dump_file.unlink()
        if self.container_file.exists():
            self.container_file.unlink()

    def write_log(self, file_path, data):
        with file_path.open('a',
                            encoding='utf-8',
                            errors='replace') as flip:
            flip.write(data)



# -*- coding: utf_8 -*-
"""Dynamic Analyzer Reporting for iOS devices."""
import logging
import shlex
import shutil
from pathlib import Path
import json

from django.conf import settings
from django.shortcuts import render
from django.template.defaulttags import register
from django.views.decorators.http import require_http_methods
from django.http import HttpResponse

import mobsf.MalwareAnalyzer.views.Trackers as Trackers
from mobsf.DynamicAnalyzer.views.ios.analysis import (
    get_screenshots,
    ios_api_analysis,
    run_analysis,
)
from mobsf.DynamicAnalyzer.views.ios.device.dynamic_analyzer import (
    FAILED,
    OK,
    validate_and_connect_device,
)
from mobsf.DynamicAnalyzer.tools.webproxy import (
    get_http_tools_url,
    stop_httptools,
)
from mobsf.MobSF.utils import (
    IOS_DEVICE_ID_REGEX,
    SSH_DEVICE_ID_REGEX,
    base64_decode,
    get_md5,
    is_md5,
    key,
    pretty_json,
    print_n_send_error_response,
    replace,
    strict_package_check,
)
from mobsf.MobSF.views.authentication import (
    login_required,
)
from mobsf.DynamicAnalyzer.views.common.shared import (
    send_response,
)
from mobsf.MobSF.views.authorization import (
    Permissions,
    permission_required,
)


logger = logging.getLogger(__name__)
register.filter('key', key)
register.filter('replace', replace)
register.filter('pretty_json', pretty_json)
register.filter('base64_decode', base64_decode)


def download_app_data_device(ios_device, checksum):
    """Download App data from device."""
    app_dir = Path(settings.UPLD_DIR) / checksum
    container_file = app_dir / 'mobsf_app_container_path.txt'
    if container_file.exists():
        app_container = container_file.read_text(
            'utf-8').splitlines()[0].strip()
        tarfile = f'/tmp/{checksum}-app-container.tar'
        localtar = app_dir / f'{checksum}-app-container.tar'
        ios_device.execute_command(
            f'tar -C {shlex.quote(app_container)} -cvf {shlex.quote(tarfile)} .')
        ios_device.download_file(tarfile, localtar)
        if localtar.exists():
            dst = Path(settings.DWD_DIR) / f'{checksum}-app_data.tar'
            shutil.copyfile(localtar, dst)

# JSON API


@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def download_data_device(request, bundle_id, api=False):
    """Download Application Data from Device."""
    logger.info('Downloading application data')
    data = {
        'status': FAILED,
        'message': 'Failed to Download application data'}
    try:
        device_id = request.POST['device_id']
        ios_device, error = validate_and_connect_device(device_id)
        if error:
            data['message'] = error
            return send_response(data, api)
        if not strict_package_check(bundle_id):
            data['message'] = 'Invalid iOS Bundle id'
            return send_response(data, api)
        checksum = get_md5(bundle_id.encode('utf-8'))
        # App Container download
        logger.info('Downloading app container data')
        download_app_data_device(ios_device, checksum)
        # Stop HTTPS Proxy
        stop_httptools(get_http_tools_url(request))
        # Move HTTP raw logs to download directory
        flows = Path.home() / '.httptools' / 'flows'
        webf = flows / f'{bundle_id}.flow.txt'
        dwd = Path(settings.DWD_DIR)
        dweb = dwd / f'{checksum}-web_traffic.txt'
        if webf.exists():
            shutil.copyfile(webf, dweb)
        data = {
            'status': OK,
            'message': 'Downloaded application data',
        }
    except Exception as exp:
        logger.exception('Downloading application data')
        data['message'] = str(exp)
    return send_response(data, api)


@login_required
def view_report_device(request, bundle_id, api=False):
    """Dynamic Analysis Report Generation."""
    logger.info('iOS Dynamic Analysis Report Generation')
    try:
        if api:
            device_id = request.POST.get('device_id')
        else:
            device_id = request.GET.get('device_id')
        if device_id:
            usb = IOS_DEVICE_ID_REGEX.match(device_id)
            wifi = SSH_DEVICE_ID_REGEX.match(device_id)
            if not (usb or wifi):
                return print_n_send_error_response(
                    request,
                    'Invalid device id or SSH connection string format',
                    api)
        if not strict_package_check(bundle_id):
            # bundle_id is not validated in REST API.
            # Also bundleid is not strictly validated
            # in URL path.
            return print_n_send_error_response(
                request,
                'Invalid iOS Bundle id',
                api)
        checksum = get_md5(bundle_id.encode('utf-8'))
        app_dir = Path(settings.UPLD_DIR) / checksum
        download_dir = settings.DWD_DIR
        tools_dir = settings.TOOLS_DIR
        frida_log = app_dir / 'mobsf_frida_out.txt'
        data_dir = app_dir / 'DYNAMIC_DeviceData'
        if not (frida_log.exists() or data_dir.exists()):
            msg = ('Dynamic Analysis report is not available '
                   'for this app. Perform Dynamic Analysis '
                   'and generate the report.')
            return print_n_send_error_response(request, msg, api)
        api_analysis = ios_api_analysis(app_dir)
        dump_analysis = run_analysis(app_dir, bundle_id, checksum)
        trk = Trackers.Trackers(checksum, app_dir, tools_dir)
        trackers = trk.get_trackers_domains_or_deps(
            dump_analysis['domains'], None)
        screenshots = get_screenshots(checksum, download_dir)
        logger.info('Report generation completed')
        context = {
            'hash': checksum,
            'device_id': device_id,
            'version': settings.MOBSF_VER,
            'title': 'iOS Dynamic Analysis Report',
            'bundleid': bundle_id,
            'trackers': trackers,
            'screenshots': screenshots,
            'frida_logs': frida_log.exists(),
        }
        context.update(api_analysis)
        context.update(dump_analysis)
        template = 'dynamic_analysis/ios/device/dynamic_report.html'
        if api:
            return context
        return render(request, template, context)
    except Exception as exp:
        logger.exception('Dynamic Analysis Report Generation')
        err = f'Error Generating Dynamic Analysis Report. {str(exp)}'
        return print_n_send_error_response(request, err, api)


# JSON API + File Download
@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def download_file_device(request, api=False):
    """Download file from device."""
    err_msg = 'Failed to download file'
    data = {
        'status': FAILED,
        'message': err_msg,
    }
    try:
        remote_path = request.POST['file']
        device_id = request.POST['device_id']
        ios_device, error = validate_and_connect_device(device_id)
        if error:
            data['message'] = error
            return send_response(data, api)
        file_obj = ios_device.download_file_object(remote_path)
        if file_obj:
            response = HttpResponse(file_obj, content_type='application/octet-stream')
            response['Content-Disposition'] = (
                f'inline; filename={Path(remote_path).name}')
            return response
        else:
            data['message'] = 'Failed to download file'
    except Exception as exp:
        logger.exception(err_msg)
        data['message'] = str(exp)
    return send_response(data, api)


# JSON API
@login_required
def ios_api_monitor(request, api=False):
    try:
        data = {
            'status': FAILED,
            'message': 'Failed to get API monitor data',
        }
        if api:
            checksum = request.POST['checksum']
            stream = True
        else:
            checksum = request.GET.get('checksum', '')
            stream = request.GET.get('stream', '')
        bundle_id = request.GET.get('bundle_id', '')
        if bundle_id and not strict_package_check(bundle_id):
            data['message'] = 'Invalid iOS Bundle id'
            return send_response(data, api)
        if not is_md5(checksum):
            data['message'] = 'Invalid checksum format'
            return send_response(data, api)
        if stream:
            app_dir = Path(settings.UPLD_DIR) / checksum
            apimon_file = app_dir / 'mobsf_dump_file.txt'
            data = {}
            if not apimon_file.exists():
                data['message'] = 'Data does not exist.'
                return send_response(data, api)
            # Read the file and parse each line as JSON
            lines = apimon_file.read_text(encoding='utf-8').splitlines()
            # remove all duplicate lines
            lines = list(set(lines))
            api_data = []

            for line in lines:
                if line.strip():
                    try:
                        # Parse each line as JSON
                        json_obj = json.loads(line)

                        # Determine the API type based on the keys in the JSON object
                        api_type = None
                        if 'cookies' in json_obj:
                            api_type = 'Cookies'
                        elif 'keychain' in json_obj:
                            api_type = 'Keychain'
                        elif 'json' in json_obj:
                            api_type = 'JSON Data'
                        elif 'network' in json_obj:
                            api_type = 'Network Request'
                        elif 'sql' in json_obj:
                            api_type = 'SQL Query'
                        elif 'filename' in json_obj:
                            api_type = 'File Access'
                        elif 'crypto' in json_obj:
                            api_type = 'Crypto'
                        elif 'files' in json_obj:
                            api_type = 'Files'
                        elif 'nslog' in json_obj:
                            api_type = 'Logs'
                        elif 'credentialstorage' in json_obj:
                            api_type = 'Credentials'
                        elif 'nsuserdefaults' in json_obj:
                            api_type = 'UserDefaults'
                        elif 'pasteboard' in json_obj:
                            api_type = 'Pasteboard'
                        elif 'textinput' in json_obj:
                            api_type = 'Text Inputs'
                        elif 'datadir' in json_obj:
                            api_type = 'Data Directory'
                        else:
                            # If no known key, use the first key as API type
                            api_type = list(json_obj.keys())[
                                0] if json_obj else 'Unknown'

                        # Create the data object for DataTable
                        api_data.append({
                            'api': api_type,
                            'data': json_obj,
                        })
                    except json.JSONDecodeError:
                        # Skip invalid JSON lines
                        continue

            data['data'] = api_data
            return send_response(data, api)
        template = 'dynamic_analysis/ios/api_monitor.html'
        return render(request,
                      template,
                      {'checksum': checksum,
                       'bundle_id': bundle_id,
                       'version': settings.MOBSF_VER,
                       'title': 'API Monitor'})
    except Exception:
        logger.exception('API monitor streaming')
        err = 'Error in API monitor streaming'
        return print_n_send_error_response(request, err, api)
